// DRAW_shell.cxx
//
// Coded August 10, 2005 by L.W. Finger, M. Kroeker, and B. Toby
//
// Part of the DRAWxtl system

// Routines contined in this module
//
//  About_Help_cb - callback to load the "About" screen
//  Color_Help_cb - callback to load the Color Help screen
//  Command_Help_cb - callback to load the Command-Line Help screen
//  Configure_DRAWxtl_cb - callback to load the "Configure DRAWxtl" browser
//  Configure_POV_cb - callback to load the "Configure POV" browser
//  Configure_VRML_cb - callback to load the "Configure VRML" browser
//  Edit_Changed_cb - callback entered when the text in the "Edit STR file screen" is changed
//  Edit_Data_cb - callback entered when the "Edit Data" menu item is selected
//  Edit_STR_Close_cb - callback entered when the "Close" button on the Edit Data Screen is pressed
//  Edit_STR_Save_cb - callback entered when the "Save" button on the Edit Data Screen is pressed
//  Error_Box - routine to pop-up an error box
//  Error_Box_cb - callback entered when the Error Box "Close" button is pressed   
//  Exit_cb - callback entered when the "Exit" menu item or the Window Close "X" is pressed
//  Graphics_Help_cb - callback entered when the "Graphics Help" menu item is selected
//  Input_Help_cb - callback entered when the "Input Help" menu item is selected
//  LoadConfig - routine to read the configuration file
//  make_window - routine to create the main-screen widgets
//  Options_Close_cb - callback entered when the "Close" button on the Options screen is pressed
//  Options_Save_cb - callback entered when the "Save" button on the Options screen is pressed
//  Run_DRAWxtl_cb - callback entered when the "Run DRAWxtl" button is pressed
//  SelectDataFile_cb - callback to load the "New Data File"browser
//  Set_Options_cb - callback to load the options screen
//  View_Help_Close_cb- callback entered when the "Close" button is pushed on one of the help screens
//  View_List_Close_cb - callback entered when the "Close" button is pushed on the listing screen
//  View_Listing_cb - callback to load the listing screen
//  View_POV_cb - callback to call the POV external renderer
//  View_VRML_cb - callback to call the external VRML renderer
//  WriteConfig - routine to update the configuration file

#include "draw_shell.h"
#ifdef WIN32
  const char* flu_file_chooser( const char *, const char *, const char * );
  char Configure_file[] = {".drawxtlrc"};
  char foldersymbol = '\\';
#else
  char Configure_file[] = {"~/.drawxtlrc"};
  char foldersymbol = '/';
#endif

Fl_Menu_Item menu_File[] = {
 {"File", 0,  0, 0, FL_SUBMENU, 0, 0, 14, 56},
 {" Select Data File", 0,  (Fl_Callback*)SelectDataFile_cb, 0, 0, 0, 0, 14, 56},
 {" Configure", 0,  0, 0, FL_SUBMENU, 0, 0, 14, 56},
 {" DRAWxtl", 0,  (Fl_Callback*)Configure_DRAWxtl_cb, 0, 0, 0, 0, 14, 56},
 {" POV", 0,  (Fl_Callback*)Configure_POV_cb, 0, 0, 0, 0, 14, 56},
 {" VRML", 0,  (Fl_Callback*)Configure_VRML_cb, 0, 0, 0, 0, 14, 56},
 {0},    // end Configure Pulldown
 {" Set Options", 0,  (Fl_Callback*)Set_Options_cb, 0, FL_MENU_DIVIDER, 0, 0, 14, 56},
 {" Exit", 0,  (Fl_Callback*)Exit_cb, 0, 0, 0, 0, 14, 56},
 {0},    // end File Pulldown
 {" Edit Data", 0,  (Fl_Callback*)Edit_Data_cb, 0, 0, 0, 0, 14, 56},
 {" Run DRAWxtl", 0,  (Fl_Callback*)Run_DRAWxtl_cb, 0, 0, 0, 0, 14, 56},
 {"View", 0,  0, 0, FL_SUBMENU, 0, 0, 14, 56},
 {" VRML", 0,  (Fl_Callback*)View_VRML_cb, 0, 0, 0, 0, 14, 56},
 {" POV", 0,  (Fl_Callback*)View_POV_cb, 0, 0, 0, 0, 14, 56},
 {" Listing", 0,  (Fl_Callback*)View_Listing_cb, 0, 0, 0, 0, 14, 56},
 {0},    // end View Pulldown  
 {"Help", 0,  0, 0, 64, 0, 0, 14, 56},
 {" Input File Commands", 0,  (Fl_Callback*)Input_Help_cb, 0, 0, 0, 0, 14, 56},
 {" Color Name/RGB Table", 0,  (Fl_Callback*)Color_Help_cb, 0, 0, 0, 0, 14, 56},
 {" Command-Line Help", 0,  (Fl_Callback*)Command_Help_cb, 0, FL_MENU_DIVIDER, 0, 0, 14, 56},
 {" Graphics Help", 0,  (Fl_Callback*)Graphics_Help_cb, 0, 0, 0, 0, 14, 56},
 {" About DRAWshell", 0,  (Fl_Callback*)About_Help_cb, 0, 0, 0, 0, 14, 56},
 {0},    // end Help Pulldown
 {0}     // end Menu
};

/******************************************************************************/
/******************************************************************************/

int main(int argc, char **argv)
{
    int i=0;

    Fl_Double_Window * mainWindow = make_window();

#if !defined (WIN32) && !defined (__APPLE__)
    #include "DRAWxtl.xpm"
#endif

#ifdef WIN32
    mainWindow->icon((char *)LoadIcon(fl_display, MAKEINTRESOURCE(IDI_ICON1)));
#else
    fl_open_display();
#if !defined(__APPLE__) 
      XpmCreatePixmapFromData(fl_display,DefaultRootWindow(fl_display),drawxtl_xpm,&icon,&shapemask,NULL);
      mainWindow->icon((char *)icon);
#endif
#endif
    Fl::args(argc, argv,i);
    LoadConfig();
    mainWindow->show();
    return Fl::run();
}

/******************************************************************************/
/******************************************************************************/

void About_Help_cb(void)
{
    static int two = 2;
// callback routine for "About Help" menu button
    if (helpwindow2) {
        helpwindow2->show();
        return;
    }
    helpwindow2 = new Fl_Window(100, 100, 500, 260,"About DRAWxtl Shell");
    helpwindow2->begin();
    helpwindow2->callback((Fl_Callback*)View_Help_Close_cb,&two);

    int y = 40;
    Fl_Text_Display * box1 = new Fl_Text_Display(20,y,460,0,"DRAWxtl Shell V4.3");
    box1->box(FL_NO_BOX);
    box1->labelsize(24);
    box1->labelcolor((Fl_Color)1);

    y += 30;
    Fl_Text_Display *box2 = new Fl_Text_Display(20,y,460,0,"Copyright C 1996-2005 by Larry Finger, Martin Kroeker and Brian Toby");
    box2->box(FL_NO_BOX);
    box2->labelsize(14);
    box2->labelcolor((Fl_Color)186);

    y += 20;
    Fl_Text_Display *box3 = new Fl_Text_Display(20,y,460,20,"Modified August 10, 2005");
    box3->box(FL_NO_BOX);
    box3->labelsize(14);
    box3->labelcolor((Fl_Color)186);

    y += 30;
    Fl_Text_Display *box4 = new Fl_Text_Display(20,y,460,20,"DRAWxtl Shell uses the FLTK 1.1.6 Widget"
        " set by ");
    box4->box(FL_NO_BOX);
    box4->labelsize(14);
    box4->labelcolor((Fl_Color)186);

    y += 20;
    Fl_Text_Display *box5 = new Fl_Text_Display(20,y,460,20,"Bill Spitzak and others, http://www.fltk.org");
    box5->box(FL_NO_BOX);
    box5->labelsize(14);
    box5->labelcolor((Fl_Color)186);
   
    y += 30;
    Fl_Text_Display *box7 = new Fl_Text_Display(20,y,460,20,"Some classes from FLU 2.14 by Jason Bryan are also employed.");
    box7->box(FL_NO_BOX);
    box7->labelsize(14);
    box7->labelcolor((Fl_Color)186);
   
    y += 20;
    Fl_Text_Display *box8 = new Fl_Text_Display(20,y,460,20,"http://www.osc.edu/~jbryan/FLU");
    box8->box(FL_NO_BOX);
    box8->labelsize(14);
    box8->labelcolor((Fl_Color)186);
   
    y += 20;
    Fl_Button* o = new Fl_Button(210, y, 80, 30, "Close");
    o->callback((Fl_Callback*)View_Help_Close_cb,&two);
#if !defined (WIN32) && !defined (__APPLE__)
    helpwindow2->icon((char *)icon);
#endif
    helpwindow2->end();
    helpwindow2->show();
}

/******************************************************************************/
/******************************************************************************/

void Color_Help_cb(void)
{
    static int one = 1;
// Callback routine to show RBG values
char * colortext = {
"This default color table is taken from the POV file 'colors.inc'\n"
"and is sorted by RGB Values - Red first, etc.. If you wish to use\n"
"custom colors, edit the above file and use the 'Configure' screen\n"
"to include the path to the file."
"\n"
"\n"
"Color Name        R             G               B\n"
"\n"
"Black		  0		0		0\n"
"Mica		  0		0		0\n"
"NewMidnightBlue	  0		0		0.61\n"
"Blue		  0		0		1\n"
"SlateBlue	  0		0.498039	1\n"
"Green		  0		1		0\n"
"SpringGreen	  0		1		0.498039\n"
"Cyan		  0		1		1\n"
"Gray05		  0.05		0.05		0.05\n"
"Gray10		  0.1		0.1		0.1\n"
"HuntersGreen	  0.13		0.37		0.31\n"
"Navy		  0.137255	0.137255	0.556863\n"
"NavyBlue	  0.137255	0.137255	0.556863\n"
"SteelBlue	  0.137255	0.419608	0.556863\n"
"ForestGreen	  0.137255	0.556863	0.137255\n"
"SeaGreen	  0.137255	0.556863	0.419608\n"
"Gray15		  0.15		0.15		0.15\n"
"MidnightBlue	  0.184314	0.184314	0.309804\n"
"DarkGreen	  0.184314	0.309804	0.184314\n"
"DarkSlateGray	  0.184314	0.309804	0.309804\n"
"DarkSlateGrey	  0.184314	0.309804	0.309804\n"
"MediumBlue	  0.196078	0.196078	0.8\n"
"SkyBlue		  0.196078	0.6		0.8\n"
"LimeGreen	  0.196078	0.8		0.196078\n"
"MediumAquamarine  0.196078	0.8		0.6\n"
"Gray20		  0.2		0.2		0.2\n"
"SummerSky	  0.22		0.69		0.87\n"
"Gray25		  0.25		0.25		0.25\n"
"CornflowerBlue	  0.258824	0.258824	0.435294\n"
"MediumSeaGreen	  0.258824	0.435294	0.258824\n"
"DkGreenCopper	  0.29		0.46		0.43\n"
"Gray30		  0.3		0.3		0.3\n"
"NeonBlue	  0.3		0.3		1\n"
"IndianRed	  0.309804	0.184314	0.184314\n"
"Violet		  0.309804	0.184314	0.309804\n"
"DarkOliveGreen	  0.309804	0.309804	0.184314\n"
"GreenCopper	  0.32		0.49		0.46\n"
"DimGray		  0.329412	0.329412	0.329412\n"
"DimGrey		  0.329412	0.329412	0.329412\n"
"VeryDarkBrown	  0.35		0.16		0.14\n"
"Gray35		  0.35		0.35		0.35\n"
"RichBlue          0.35          0.35            0.67\n"
"BakersChoc	  0.36		0.2		0.09\n"
"DarkBrown	  0.36		0.25		0.2\n"
"CadetBlue	  0.372549	0.623529	0.623529\n"
"Gray40		  0.4		0.4		0.4\n"
"DarkSlateBlue	  0.419608	0.137255	0.556863\n"
"MediumForestGreen 0.419608	0.556863	0.137255\n"
"SemiSweetChoc	  0.42		0.26		0.15\n"
"Salmon		  0.435294	0.258824	0.258824\n"
"DarkTurquoise	  0.439216	0.576471	0.858824\n"
"Aquamarine	  0.439216	0.858824	0.576471\n"
"MediumTurquoise	  0.439216	0.858824	0.858824\n"
"Gray45		  0.45		0.45		0.45\n"
"MediumSlateBlue	  0.498039	1		0\n"
"MediumSpringGreen 0.498039	0		1		\n"
"Gray50		  0.5		0.5		0.5\n"
"DarkWood	  0.52		0.37		0.26\n"
"DustyRose	  0.52		0.39		0.39\n"
"DarkPurple	  0.53		0.12		0.47\n"
"Scarlet		  0.55		0.09		0.09\n"
"Bronze		  0.55		0.47		0.14\n"
"Gray55		  0.55		0.55		0.55\n"
"Firebrick	  0.556863	0.137255	0.137255\n"
"Maroon		  0.556863	0.137255	0.419608\n"
"Sienna		  0.556863	0.419608	0.137255\n"
"LightSteelBlue	  0.560784	0.560784	0.737255\n"
"PaleGreen	  0.560784	0.737255	0.560784\n"
"MediumOrchid	  0.576471	0.439216	0.858824\n"
"GreenYellow	  0.576471	0.858824	0.439216\n"
"DarkTan		  0.59		0.41		0.31\n"
"DarkOrchid	  0.6		0.196078	0.8\n"
"Gray60		  0.6		0.6		0.6\n"
"YellowGreen	  0.6		0.8		0.196078\n"
"BlueViolet	  0.62352	0.372549	0.623529\n"
"\n"
"Khaki		  0.623529	0.623529	0.372549\n"
"Brown		  0.647059	0.164706	0.164706\n"
"Bronze2		  0.65		0.49		0.24\n"
"MediumWood	  0.65		0.5		0.39\n"
"Gray65		  0.65		0.65		0.65\n"
"LightGray	  0.658824	0.658824	0.658824\n"
"LightGrey	  0.658824	0.658824	0.658824\n"
"Turquoise	  0.678431	0.917647	0.917647\n"
"Gray70		  0.7		0.7		0.7\n"
"Brass		  0.71		0.65		0.26\n"
"Copper		  0.72		0.45		0.2\n"
"Pink		  0.737255	0.560784	0.560784\n"
"LightBlue	  0.74902	0.847059	0.847059\n"
"Gray75		  0.75		0.75		0.75\n"
"Gray		  0.752941	0.752941	0.752941\n"
"Grey		  0.752941	0.752941	0.752941\n"
"VioletRed	  0.8		0.196078	0.6\n"
"Gold		  0.8		0.498039	0.196078\n"
"Gray80		  0.8		0.8		0.8\n"
"VLightGrey	  0.8		0.8		0.8\n"
"OldGold		  0.81		0.71		0.23\n"
"Feldspar	  0.82		0.57		0.46\n"
"Thistle		  0.847059	0.74902		0.847059\n"
"Wheat		  0.847059	0.847059	0.74902\n"
"CoolCopper	  0.85		0.53		0.1\n"
"BrightGold	  0.85		0.85		0.1\n"
"Gray85		  0.85		0.85		0.85\n"
"Quartz		  0.85		0.85		0.95\n"
"MediumVioletRed	  0.858824	0.439216	0.576471\n"
"Orchid		  0.858824	0.439216	0.858824\n"
"Tan		  0.858824	0.576471	0.439216\n"
"Goldenrod	  0.858824	0.858824	0.439216\n"
"MandarinOrange	  0.89		0.47		0.2\n"
"Gray90		  0.9		0.9		0.9\n"
"Silver		  0.9		0.91		0.98\n"
"LightWood	  0.91		0.76		0.65\n"
"Plum		  0.917647	0.678431	0.917647\n"
"MediumGoldenrod	  0.917647	0.917647	0.678431\n"
"NewTan		  0.92		0.78		0.62\n"
"Gray95		  0.95		0.95		0.95\n"
"Flesh		  0.96		0.8		0.69\n"
"Red		  1		0		0\n"
"Magenta		  1		0		1\n"
"SpicyPink	  1		0.11		0.68\n"
"NeonPink	  1		0.43		0.78\n"
"Coral		  1		0.498039	0\n"
"OrangeRed	  1		0.498039	0\n"
"Orange		  1		0.5		0\n"
"Yellow		  1		1		0\n"
"Clear		  1		1		1\n"
"White		  1		1		1"};

    if (helpwindow1) {
        helpwindow1->show();
        return;
    }
    helpwindow1 = new Fl_Window(50, 50, 680, 550, "Color RGB Values");
    helpwindow1->begin();
    helpwindow1->callback((Fl_Callback*)View_Help_Close_cb, &one);
    Fl_Text_Display * display = new Fl_Text_Display(0, 0, 680, 510);
    display->textsize(13);
    display->textfont(FL_COURIER);
    Fl_Text_Buffer * m = new Fl_Text_Buffer();
    display->buffer(m);
    m->text(colortext);
    Fl_Button* o = new Fl_Button(300, 515, 80, 30, "Close");
    o->callback((Fl_Callback*)View_Help_Close_cb, &one);
#if !defined (WIN32) && !defined (__APPLE__)
    helpwindow1->icon((char *)icon);
#endif
    helpwindow1->end();
    helpwindow1->show();
}

/******************************************************************************/
/******************************************************************************/

void Command_Help_cb(void)
{
    static int four = 4;
    char Help_Text[] = {
"            Command Line Options\n"
"-a\n"
"   sets the program to label each atom in the asymmetric unit when reading from SHELX files.\n"
"\n"
"-b xlim ylim zlim\n"
"     defines X-, Y- and Z-limits of the view box in angstroms (defaults to -10..10 Angstrom\n"
"     around center)\n"
"\n"
"-c \n"
"   expands the VRML output by adding comments. This option should not be used for any VRML\n"
"   files placed in a Web site.\n"
"\n"
"-C\n"
"   switches the VRML camera to orthographic, rather than perspective mode.\n"
"\n"
"-h, -?\n"
"   displays usage information\n"
"\n"
"-m factor\n"
"   changes the magnification factor for POV and VRML diagrams in case the initial size is\n"
"   not correct (defaults to 1.0)\n"
"\n"
"-n\n"
"   suppresses plotting of unit cell 'box'\n"
"\n"
"-o xcenter ycenter zcenter\n"
"   defines center of view box in crystal coordinates (defaults to 0.5 0.5 0.5)\n"
"\n"
"-O  is the same as -C\n"
"\n"
"-p xmin xmax ymin ymax zmin zmax   \n"
"   defines a-,b-,c- plotting range in fractions of the axes, similar to the Pluto\n"
"   (Motherwell & Clegg 1978) PACK RANGE command (this is especially useful for hexagonal\n"
"   cells, where the orthorhombic view box does not always give satisfactory results)\n"
"\n"
"-v xangle yangle zangle\n"
"   defines view rotation angles around the axes of the Cartesian view coordinate system\n"
"   (equivalent to the view keyword) (defaults to 0 0 0)\n"
"\n"
"-u\n"
"   display a orientation vector triple at the edge of the diagram. If this switch is not\n"
"   used, the axes are labeled.\n"
"\n"
"If command-line and input-file information for the same quantities are provided, the\n"
"command-line values will control the resulting drawing and override the input-file values.\n"};

    if (helpwindow4) {
        helpwindow4->show();
        return;
    }

    helpwindow4 = new Fl_Window(50, 50, 680, 630, "Command-Line Options");
    helpwindow4->begin();
    helpwindow4->callback((Fl_Callback*)View_Help_Close_cb, &four);
    Fl_Text_Display * display = new Fl_Text_Display(0, 0, 680, 590);
    display->textsize(12);
    display->textfont(FL_COURIER_BOLD);
    Fl_Text_Buffer * m = new Fl_Text_Buffer();
    display->buffer(m);
    m->text(Help_Text);
    Fl_Button* o = new Fl_Button(300, 595, 80, 30, "Close");
    o->callback((Fl_Callback*)View_Help_Close_cb, &four);
#if !defined (WIN32) && !defined (__APPLE__)
    helpwindow4->icon((char *)icon);
#endif
    helpwindow4->end();
    helpwindow4->show();
}

/******************************************************************************/
/******************************************************************************/

void Configure_DRAWxtl_cb(void)
{
// Callback routine to select the DRAWxtl executable

#ifdef WIN32
    const char * newfile = flu_file_chooser("Select DRAWxtl Executable",
        "*.exe", DRAWxtl_Path);
#else
    char * newfile = fl_file_chooser("Select DRAWxtl Executable", "*", DRAWxtl_Path);
#endif
    if(newfile){
        strcpy(DRAWxtl_Path,newfile);
        WriteConfig();
    }
}

/******************************************************************************/
/******************************************************************************/

void Configure_POV_cb(void)
{
// Callback routine to select the POV renderer executable

#ifdef WIN32
    const char * newfile = flu_file_chooser("Select POV Executable",
        "*.exe", POV_Path);
#else
    char * newfile = fl_file_chooser("Select POV Executable", "*", POV_Path);
#endif
    if(newfile){
        strcpy(POV_Path,newfile);
        WriteConfig();
    }
}

/******************************************************************************/
/******************************************************************************/

void Configure_VRML_cb(void)
{
// Callback routine to select the VRML renderer executable

#ifdef WIN32
    const char * newfile = flu_file_chooser("Select VRML Executable",
        "*.exe", VRML_Path);
#else
    char * newfile = fl_file_chooser("Select VRML Executable", "*", VRML_Path);
#endif
    if(newfile){
        strcpy(VRML_Path,newfile);
        WriteConfig();
        if(fl_ask("Is your VRML Viewer a browser such as Firefox, etc?"))
            strcpy(VRML_Use_Browser,"yes");
        else
            strcpy(VRML_Use_Browser,"no");
    }
}

/******************************************************************************/
/******************************************************************************/

void Edit_Changed_cb(int, int nInserted, int nDeleted,int, const char*, void* v)
{
  char title[128];
  Fl_Window *w = (Fl_Window *)v;
  if ((nInserted || nDeleted) && !Edit_loading) {
      Edit_changed = 1;
      strcpy(title,Edit_title);
      strcat(title," (modified)");
      w->label(title);
  }
}

/******************************************************************************/
/******************************************************************************/

void Edit_Data_cb(void)
{
    static int one = 1;
    static int mone = -1;
    static int zero = 0;

    if (!strlen(Cur_File)) {            // Make sure rendering enabled
        Error_Box("A Structure File must be selected first.");
        return;
    }
// callback routine to display Edit Str File screen
    strcpy(Edit_title,"Edit STR File");
    Edit_changed = 0;
    Edit_loading = 1;
    if (!textwindow) {
        textwindow = new Fl_Window(20, 20, 480, 550, Edit_title);
        textwindow->begin();
        textwindow->callback((Fl_Callback*)Edit_STR_Close_cb);
        Fl_Text_Editor * display = new Fl_Text_Editor(0, 0, 480, 510);
        display->textfont(FL_COURIER);
        display->textsize(12);
        textbuf = new Fl_Text_Buffer;
        display->buffer(textbuf);
        textbuf->add_modify_callback(Edit_Changed_cb,textwindow);
        textbuf->call_modify_callbacks();

        Fl_Button* o = new Fl_Button(50, 515, 80, 30, "Close");
        o->tooltip("Close this window and discard all changes.");
        o->callback((Fl_Callback*)Edit_STR_Close_cb);
        Fl_Button* oo = new Fl_Button(150, 515, 80, 30, "Apply");
        oo->tooltip("Apply current contents to drawing, but leave this window open.");
        oo->callback((Fl_Callback*)Edit_STR_Save_cb, &mone);
        Fl_Button* po = new Fl_Button(250, 515, 80, 30, "Save");
        po->tooltip("Apply current contents to drawing, then close this window.");
        po->callback((Fl_Callback*)Edit_STR_Save_cb, &zero);
        Fl_Button* op = new Fl_Button(350, 515, 80, 30, "Save As");
        op->callback((Fl_Callback*)Edit_STR_Save_cb, &one);
        op->tooltip("Rewrite current contents to new file, and close this window.");
#if !defined (WIN32) && !defined (__APPLE__)
        textwindow->icon((char *)icon);
#endif
        textwindow->end();
    }
    textbuf->loadfile(Cur_File);
    Edit_loading = 0;
    textwindow->show();
}

/******************************************************************************/
/******************************************************************************/

void Edit_STR_Close_cb(void)    // callback to destruct editor window
{
    static int zero = 0;
// callback to close the STR edit screen
    if (Edit_changed) {                     // buffer changed - query user
       int r = fl_choice("The current file has been modified, but not saved.\n"
                    "Would you like to save it now?",
                    "Cancel", "Yes", "No");

       if (r == 1) {
           Edit_STR_Save_cb(NULL, & zero);
           return;
       }
       if (r == 0) return;
    }
    textwindow->hide();
}

/******************************************************************************/
/******************************************************************************/

void Edit_STR_Save_cb(Fl_Button *,int * action)    // callback to save edited str file
{
// callback routine to save edit parameter data
    FILE * file;
    char * selection;

    if (*action > 0) { // true for 'Save As'
#ifdef WIN32
        char drive[_MAX_DRIVE];
        char dir[_MAX_DIR];
        char fname[_MAX_FNAME];
        char ext[_MAX_EXT];
        const char * newfile = flu_file_chooser("Select New Name for Data/Experiment File",
            "*.str", Cur_File);
        if(newfile){
            _splitpath( newfile, drive, dir, fname, ext );  //Windows code
            strcpy(Cur_Dir,drive);                // Drive letter
            strcat(Cur_Dir,dir);                  //   and directory
            strcpy(Cur_File,fname);               // copy file name
            strcat(Cur_File,ext);                 // and add extension
#else
        int i,k = 0;
        char * newfile = fl_file_chooser("Select New Name for Data/Experiment File", "*.str",
            Cur_File);
        if(newfile){
            strcpy(Cur_File,newfile);
            strcpy(Cur_Dir,newfile);
            for (i=strlen(Cur_Dir);i>0;--i) {     // Find final / in file name
                if (Cur_Dir[i-1] == '/') {
                    Cur_Dir[i] = 0;
                    break;
                }
            }
            for (i=strlen(Cur_Dir);newfile[i] != 0;i++) {
                Cur_File[k++] = newfile[i];       // copy file name to Cur_File
            }
            Cur_File[k] = 0;
#endif
            chdir(Cur_Dir);                       // switch to new directory
            if ((file = fopen(Cur_File,"r"))) {
                if(!fl_ask("Selected file exists. Do you wish to overwrite it?")) {
                    return;
                }
            }
            WriteConfig();
        }
    }      // end of *action > 0
    file = fopen(Cur_File,"w");    // open new or existing file for writing
    if (!file) {
        Error_Box("Unable to open STR file for writing");
        return;
    }

    selection = textbuf->text();
    fprintf(file,"%s",selection);  // copy text from window to file
    free(selection);
    fclose(file);                         // close the file
    Edit_changed = 0;
    if (* action >= 0){
      textwindow->hide();
    }
    Fl::redraw();
}

/******************************************************************************/
/******************************************************************************/

void Error_Box(char *message)
{
// routine for Error message windows

    if (!errorbox)
        errorbox = new Fl_Window(200, 180, 500, 80,"Error");
    errorbox->begin();
    errorbox->callback((Fl_Callback*)Error_Box_cb);
    
    int y = 20;
    Fl_Multiline_Output * box = new Fl_Multiline_Output(0,10,500,30,"");
    box->box(FL_NO_BOX);
    box->labelsize(14);
    box->labelcolor((Fl_Color)1);
    box->color(FL_GRAY);
    box->value(message);
    y += 15;
    Fl_Button* o = new Fl_Button(210, y, 80, 30, "Close");
    o->callback((Fl_Callback*)Error_Box_cb);
#if !defined (WIN32) && !defined (__APPLE__)
    errorbox->icon((char *)icon);
#endif
    errorbox->end();
    errorbox->show();
    errorbox->set_modal();
}

/******************************************************************************/
/******************************************************************************/

void Error_Box_cb(Fl_Widget * w, void *d)    
{
// callback to hide error message window
    w->window()->hide();
}

void Exit_cb(void)
{
    exit(0);
}

/******************************************************************************/
/******************************************************************************/

void Graphics_Help_cb(void)
{
    static int three = 3;
// routine to display graphics help
    if (helpwindow3) {
        helpwindow3->show();
        return;
    }
    char Text[] = {"\n                                   Graphics Keyboard Shortcuts and Help\n\n"
"    'C' or 'c' - turn the graphics cursor on. Each succesive press reduces the size of the\n"
"                   steps. When the size is reduced below 0.01 A, the cursor is turned off.\n"
"                   When atoms are selected as described below, the various distances, angles,\n"
"                   and the torsion angle 1-2-3-4 are shown in the display area.\n\n"
"    'O' or 'o' - Toggle between orthographic and perspective projection.\n\n"
"    'x', 'y', 'z' - move the cursor in the positive direction parallel to the x-, y- or z-axis.\n\n"
"    'X', 'Y', 'Z' - move the cursor in the negative direction parallel to the x-, y- or z-axis.\n\n"
"    'P' or 'p' - place the graphics cursor on the atom nearest the mouse position.\n\n"
"    'A' or 'a' - place the cursor at the position of the atom nearest the cursor.\n\n"
"    'M' or 'm' - move the cursor to the min (M) or max (m) in the electron-density.\n\n"
"    'L' or 'l' - label the atom at the cursor position.\n\n"
"    'B' or 'b' - Label the bond distance between atoms 1 and 2.\n\n"
"    Left mouse and drag - rotate the graphics object using a virtual trackball.\n\n"
#if !defined(__APPLE__)
"    Right mouse and drag - zoom in/out.\n\n"
"    Middle mouse (both on 2-button mouse) and drag - pan motion.\n"
#else
"    Command (Apple) key and drag - zoom in/out.\n\n"
"    Alt (option) key and drag - pan the graphics view.\n"
#endif
"                       The arrow keys may also be used to move the object.\n\n"
"    HOME key - remove all zoom and pan motions.\n\n"
"    Shift/leftclick - drag labels or the triple vector to a desired position. If the triple\n"
"                        vector is dragged, all 3 of its labels are moved with it.\n\n"
"    Ctrl/leftclick - remove the object at the position of the mouse.\n"
    };
    helpwindow3 = new Fl_Window(200, 50, 600, 680,"DRAWxtl V5.1 Screen Graphics Help");
    helpwindow3->begin();
    helpwindow3->callback((Fl_Callback*)View_Help_Close_cb, &three);
    int y = 40;
    Fl_Multiline_Output * a = new Fl_Multiline_Output(0, 0, 600, 610);
    a->textsize(13);
    a->box(FL_FLAT_BOX);
    a->textfont(FL_HELVETICA_BOLD);
    a->value(Text);
    a->color(FL_WHITE);
    y = 630;
    Fl_Button* o = new Fl_Button(260, y, 80, 30, "Close");
    o->callback((Fl_Callback*)View_Help_Close_cb,&three);
    o->take_focus();
#if !defined (WIN32) && !defined (__APPLE__)
    helpwindow3->icon((char *)icon);
#endif
    helpwindow3->end();
    helpwindow3->show();
}

/******************************************************************************/
/******************************************************************************/

void LoadConfig(void)
{
// load configuration file
    FILE *fname;
    char filename[256];
    char temp[256];
    int i;
    
#ifdef WIN32
    char profile[100];
    OSVERSIONINFOEX osvi;
	
    memset(&osvi, 0, sizeof(OSVERSIONINFOEX));
    osvi.dwOSVersionInfoSize = sizeof(OSVERSIONINFOEX);
    GetVersionEx ((OSVERSIONINFO *)&osvi);
	if (osvi.dwMajorVersion == 4)
		strcpy(profile,"c:");
	else
	    strcpy(profile,getenv("USERPROFILE"));
    strcpy(filename,profile);
    strcat(filename,"\\");
    strcat(filename,Configure_file);
#else
    fl_filename_expand(filename,Configure_file);
#endif
    fname = fopen(filename,"r");
    if (!fname) {
        strcpy(Cur_Dir,"Unable to open Configuration - Please use 'Configure' menu item.");
        strcpy(Cur_File,"");
        Current_Directory->value(Cur_Dir);
        return;
    }
    fgets(DRAWxtl_Path, 256, fname);
    trim_string(DRAWxtl_Path,256);
    fgets(POV_Path, 256, fname);
    trim_string(POV_Path,256);
    fgets(VRML_Path, 256, fname);
    trim_string(VRML_Path,256);
    fgets(VRML_Use_Browser, 256, fname);
    trim_string(VRML_Use_Browser,256);
    fgets(FileViewName, 256, fname);
    trim_string(FileViewName,256);
    fgets(POV_Option, 256, fname);
    trim_string(POV_Option,256);
    fgets(Cur_File, 256, fname);
    trim_string(Cur_File,256);
    fgets(Cur_Dir, 256, fname);
    trim_string(Cur_Dir,256);
    fgets(POV_Include, 256, fname);
    trim_string(POV_Include,256);
    fgets(LoadOnStartup,10,fname);
    trim_string(LoadOnStartup,10);
    fgets(DefaultFinish,256,fname);
    trim_string(DefaultFinish,256);
    fgets(GUIExePath,256,fname);
    trim_string(GUIExePath,256);
    fgets(temp,12,fname);
    if (!strncmp(temp,"autolabel",9)) {
      autolabel=1;
      strcpy(DRAWxtl_Option,"-a");
    } else {
      autolabel=0;
      strcpy(DRAWxtl_Option,"");
    }
    fclose(fname);
    if (!strcmp(FileViewName,"less") || !strcmp(FileViewName,"notepad"))
        strcpy(FileViewName,DRAWxtl_Option);
    else {
        if (!strstr(FileViewName,"-a"))
            strcat(FileViewName,DRAWxtl_Option);
    }
    strcpy(DRAWxtl_Option, FileViewName);
    Command_Line_Options->value(DRAWxtl_Option);
    if (!strstr(VRML_Use_Browser,"yes") && !strstr(VRML_Use_Browser,"no"))
        strcpy(VRML_Use_Browser,"no");
#ifndef WIN32
    if(!strlen(POV_Option)) {                    // generate plausible POV options
        strcpy(POV_Option,"+W600 +H600 +D +FC +A0.3 +L/usr/local/share/povray-3.5/include +P");
        WriteConfig();
    }
#endif
    Current_Directory->value(Cur_Dir);
    chdir(Cur_Dir); 
    Current_Structure_File->value(Cur_File);
    POV_Options->value(POV_Option);
    strcpy(Cur_Root,Cur_File);
    for (i=strlen(Cur_Root); i>0; --i) {
        if (Cur_Root[i] == '.') {
            Cur_Root[i] = '\0';
            break;
        }
    }
    strcpy(Cur_Listing,Cur_Root);
    strcat(Cur_Listing,".out");

}

/******************************************************************************/
/******************************************************************************/

void Input_Help_cb(void)
{
// callback routine for help on input - from menu item
char * helptext = {
"Input Instructions for DRAWxtl:\n"
"\n"
"In this program, color is represented in symbolic form, and must be one of the color names\n"
"from POV's 'colors.inc' file. The color names, sorted by RGB values, are available in the colors\n"
"entry under the Help menu. Any of these colors may be made transparent by appending the phrase\n"
"'filter xx' after it, where xx is a number from 0.0 to 1.0. (The larger the value of xx, the more\n"
"transparent will be the entity.\n"
"\n"
"Each line of the input file is preceded by a character sequence that describes the type of\n"
"information, as follows:\n"
"\n"
"\n"
" arrow       xp yp zp xc yc zc length diameter color\n"
"\n"
"defines the position in fractional coordinates (xp, yp, zp) of the triclinic, nuclear cell, the\n"
"components (xc, yc, zc) of the spin vector, and the length diameter and color of the arrow.. The\n"
"reference direction for xc is parallel to direct space axis a, yc is parallel (a x b) x a,\n"
"and the reference direction for zc is perpendicular to xc and yc. The only space-group symmetry\n"
"elements used in placing arrows are the translations described by the mag_trans command below.\n"
"\n"
"\n"
" atom           name number x y z\n"
"\n"
"defines the atoms. The 1- to 4-character name will be used on the commands that describe the\n"
"objects to be created, the number is to identify which atom of this type, and x y z are the\n"
"fractional coordinates in the unit cell. If the position is more easily represented as a fraction\n"
"such as 1/3, 1/4, 5/6, etc., it may be given in this form.\n"
"\n"
"\n"
"axislines    width color\n"
"\n"
"defines the width and color of the lines that depict the principal axes of ellipsoids. The color\n"
"defaults to dark gray (Gray20). If this is not given, any ellipsoids will be drawn with principal\n"
"axes of 0.00015 times the overall scale factor, which should normally be appropriate.\n"
"\n"
"\n"
" background color\n"
"\n"
"sets the color of the background of the graphical views. The default color is white.\n"
"\n"
"\n"
" betaij         name number color\n"
"\n"
"defines the anisotropic thermal coefficients for an atom and color of the ellipsoid. The name and\n"
"number should correspond to the atom input described above. In the POV version of the program, the\n"
"principal ellipses are drawn in black. The ellipses will, of course, be invisible if the ellipsoid\n"
"is also black.\n"
"\n"
"\n"
" bij or Bij     name color\n"
"\n"
"defines the anisotropic thermal coefficients for an atom and the color of the ellipsoid to be drawn.\n"
"The name and number should correspond to the atom input described above.\n"
"\n"
"\n"
" bond           name1 name2 radius min max color\n"
"\n"
"where name1 and name2 indicate the types of atoms to be connected by a bond, radius is the radius\n"
"of the resulting cylinder, and the minimum and maximum lengths are given in the same units as the\n"
"unit cell.\n"
"\n"
"\n"
" box            radius color\n"
"\n"
"defines the radius and color of the cylinders that form the unit cell boundary. If radius is 0.0,\n"
"plotting of the unit cell is suppressed. The radius of the cylinders will be scaled with the size\n"
"of the drawing. The default size is 0.02.\n"
"\n"
"\n"
" cell           a b c alpha beta gamma\n"
"\n"
"unit-cell lengths and angles. If no angles are listed, they are assumed to be the fixed values for\n"
"the symmetry class.\n"
"\n"
"\n"
" clip          xmin xmax ymin ymax zmin zmax\n"
"\n"
"defines a-,b-,c- clipping range in fractions of the axes. Any bonds extending beyond these limits\n"
"will be cut off at half-length. This command is to be used in conjunction with the pack keyword to\n"
"produce 'dangling' bonds in the display of framework structures.\n"
"\n"
"\n"
" cutout         color     (used only for POV and openGL)\n"
"\n"
"sets the POV generation of thermal ellipsoids to have one octant removed, as in the program ORTEP.\n"
"If this command is not given, all ellipsoids will be complete. The color is for the planes that\n"
"describe the edges of the cutout.\n"
"\n"
"\n"
" dash           name1 name2 radius min max color\n"
"\n"
"where name1 and name2 indicate the types of atoms to be connected by a dashed bond, radius is the\n"
"radius of the resulting cylinder, and the minimum and maximum lengths are given in the same units as the\n"
"unit cell.\n"
"\n"
"\n"
" depthcue       depth (POV and openGL only)\n"
"\n"
"defines the extent to which the size of polyhedral edges is increased as the edge is closer to the\n"
"viewer.\n"
"\n"
"\n"
" edges          radius color\n"
"\n"
"defines the thickness and color of cylinders along the edges of polyhedra that may be used to\n"
"emphasize the faces. The radius of these cylinders will also be scaled with the size of the drawing.\n"
"By default, black edges of size 0.02 will be drawn.\n"
"\n"
"\n"
" ellipcolor     name number color\n"
"\n"
"defines the color for ellipsoids when the thermal ellipsoid information has been read from a CIF,\n"
"GSAS, SCHAKAL or SHELX import or inline file. The name and number must match the identification\n"
"information in the input file. The parameter number may be an asterisk (*) to indicate all atoms\n"
"with that name. In addition, these input lines must be after the import or inline command.\n"
"\n"
"\n"
" ellipsoids     probability\n"
"\n"
"sets the size of the ellipsoid such that that fraction of the electron density is contained within\n"
"the bounding surface. Use either 0.50 or 50 to get the standard (default) 50% ellipsoids.\n"
"\n"
"\n"
" finish        ambient diffuse specular roughness\n"
"\n"
"defines parameters for the POV lighting functions that are applied to\n"
"all surfaces. Suggested values are 0.7 0.3 0.08 0.01 to reduce the harsh\n"
"contrasts that can result from the default material properties in POV.\n"
"\n"
"\n"
" frame        comment\n"
"\n"
"similar to 'end', marks the division between two complete sets of input that are to be superimposed\n"
"in a single output file. Use this where you would otherwise have to create separate datafiles (e.g.\n"
"one showing polyhedra, the other selected bonds) and join the resulting POV or VRML files.\n"
"For V4.1 and later, each frame may have different space groups, pack range, and object descriptions.\n"
"With this functionality, a single drawing can represent a cage-compound framework with an adsorbed\n"
"molecule that has lower symmetry than the framework. Another option is to draw ball-and-stick and\n"
"polyhedral pictures in side-by-side unit cells.\n"
"\n"
"\n"
" import         cif filename\n"
" import         csd filename (or fdat filename)  causes the program to read\n"
" import         gsas filename phasenumber        information from an external file.\n"
" import         schakal filename                 'filename' is the name of the file.\n"
" import         shelx filename\n"
" import         wien2k filename\n"
"\n"
"Import filters have been written for the CIF, FDAT (Cambridge Structure Database=CSD), GSAS,\n"
"SCHAKAL, SHELX and WIEN2k formats.  For GSAS format, the number of the phase should also be given.\n"
"From these files, the atomic coordinates, thermal parameters, unit cell, and space group will be read.\n" 
"To turn on ellipsoid output, and to set colors for the ellipsoids, use the ellipcolor command.\n" 
"\n"
"\n"
" inline          cif\n"
" inline          csd (or fdat)\n"
" inline          schakal\n"
" inline          shelx\n"
" inline          wien2k\n"
"\n"
"is similar to import, except that the foreign input information is included in the DRAWxtl input file\n"
"in the lines immediately following this command. This form presently works for CIF, FDAT, CIF,\n"
"SCHAKAL, SHELX and WIEN2k data. To turn on ellipsoid output, and to set colors for the ellipsoids,\n" 
"use the ellipcolor command.\n"
"\n"
"\n"
" labelsize size\n"
"\n"
"changes the relative size of 'labeltext' entries. The default is 1.0\n"
"\n"
"\n"
" labeltext x y z text\n"
"\n"
"places the text at position x,y,z\n"
"\n"
"\n"
" list maxdist\n"
"\n"
"causes the program to list bond distances up to 'maxdist' in the preliminary scan. If this command is\n"
"not given, 'maxdist' defaults to 3.5 Angstrom.\n"
"\n"
"\n"
" lonepair name number height radius1 radius2 color\n"
"\n"
"creates the specified 'number' (either 1 or 2) of cones representing free electron pairs extending from\n"
"atom 'name', where 'height' is the length of the cone, 'radius1' is the size of the tip, and 'radius2'\n"
"is the size of the spherical end cap.\n"
"\n"
"\n"
" mag_trans      Aa Ab Ac Ba Bb Bc Ca Cb Cc\n"
"\n"
"describes the relationship between the magnetic and nuclear unit cells. In this notation, the\n"
"upper-case letter states which of the magnetic axes is being described, and the lower-case\n"
"letter corresponds to the nuclear cell axis. This matrix defaults to the identity.\n"
"\n"
"\n"
" magnification  factor\n"
"\n"
"sets the factor to modify the overall scaling is case the automatic value is not correct.\n"
"\n"
"\n"
" mapcalclimits xmin xmax ymin ymax zmin zmax\n"
"\n"
"describes the region of direct space (in fractional coordinates) for which the map has been calculated.\n"
"Map types that are self documenting such as FullProf and JANA2000 do not need this line. For other\n"
"types, 0 to 1 in all three directions will be assumed.\n"
"\n"
"\n"
" mapcontour level style color\n"
"\n"
"defines a new contour at 'level'. The style can be either 'mesh' or 'solid', and the color is\n"
"set by 'color'\n"
"\n"
"\n"
" mapread  maptype filename calctype\n"
"\n"
"reads a Fourier map of type 'maptype' from the file named 'filename'. At present, GSAS-style\n"
"(maptype = grd), JANA2000-style (maptype = stf), WIEN2k (maptype = w2k), VASP (maptype = vsp), and\n"
"FullProf (maptype = flp) maps are read.\n"
"If a Shelx/Cif-style Fo/Fc file (maptype = fcf) is given, the electron density is calculated\n"
"during the initial read, which may take a few seconds. Both A/B and Fo/phi data formats\n" 
"(Shelx commands LIST 3 and LIST 6) are supported. The calctype may be 'Fo', 'Fc', 'Fo-Fc', or '2Fo-Fc'\n"
"to indicate the type of map to calculate. If this parameter is not given, an 'Fo' map is calculated.\n"
"\n"
"\n"
" mapregion xmin xmax ymin ymax zmin zmax\n"
"\n"
"describes the region of direct space (in fractional coordinates) that the map is to be displayed\n"
"in the output. If not entered, these values default to the values given under 'mapcalclimits'.\n"
"\n"
"\n"
" molcomp        dist\n"
"\n"
"causes any incomplete molecules in the display box to be completed. The value of dist defines the\n"
"maximum intramolecular distance. Caution: If this distance is greater than any intermolecular distance,\n"
"or if the material is not molecular, the display list will overflow.\n"
"\n"
"\n"
" nolabels\n"
"\n"
"removes all axis labels from the output diagrams.\n"
"\n"
"\n"
" noshadow\n"
"\n"
"causes objects in the POV file not to cast shadows.\n"
"\n"
"\n"
" origin         xcenter ycenter zcenter\n"
"\n"
"defines center of view box in crystal coordinates (defaults to 0.5 0.5 0.5).\n"
"\n"
"\n"
" orthographic\n"
"\n"
"causes the camera to be changed from the normal perspective view to an orthographic view.\n"
"\n"
"\n"
" pack           xmin xmax ymin ymax zmin zmax\n"
"\n"
"defines a-,b-,c- plotting range in fractions of the axes, similar to the PLUTO (Motherwell & Clegg 1978))\n"
"PACK RANGE command (this is especially useful for highly oblique cells, where the orthorhombic\n"
"view box does not always give satisfactory results). When drawing multiple frames, each frame can\n"
"have distinct packing values. If new values are not defined, they will be derived from the previous\n"
"frame.\n"
"\n"
"\n"
" phong          value size                (used only for POV and openGL)\n"
"\n"
"defines the amount of Phong highlighting on spheres and ellipsoids.  The value ranges between 0.0\n"
"and 1.0, where 0.0 gives no highlight, and 1.0 causes complete saturation at the center of the highlight.\n"
"The size ranges from 1.0 (very dull) to 250 highly) polished). The default quantities are 0.1 and 1.0,\n"
"which gives a large, dull highlight.  If value is 0.0, the image can be rendered much more quickly.\n"
"\n"
"\n"
" plane          name length color\n"
"\n"
"defines the center of a plane group, such as CO3 that is to be drawn in a structure, where name is the\n"
"name of the atom at the center and length is the maximum distance to coordinating anions.\n"
"\n"
"\n"
" polyedge name radius color\n"
"\n"
"defines the thickness and color of cylinders used to emphasize the faces along the edges of polyhedra\n"
"for atom 'name'. The radius of these cylinders will also be scaled with the size of the drawing.\n"
"\n"
"\n"
" polysz         name length color\n"
"\n"
"defines a polyhedron, where name is the name of an atom at the center of a polyhedron and length is\n"
"the maximum length of distances to atoms that are to be considered as the vertices of the polyhedron.\n"
"The polyhedra can be of any desired complexity. For polyhedra with both upper and lower limits\n"
"(which might be desirable for intermetallic compounds), use the 'shell' command. To control both\n"
"center and target atoms, use the 'polyvert' command\n"
"\n"
"\n"
" polytolerance      factor\n"
"\n"
"modifies the internal limit for the deviation of vertices from the common plane. While the default\n"
"value (0.1) will always generate correct drawings, it may sometimes be desirable to increase it to\n"
"create idealized views of nearly symmetrical polyhedra that would otherwise show creased surfaces.\n"
"\n"
"\n"
" polyvert       name1 name2 length color\n"
"\n"
"defines a polyhedron, where atoms of type name1 are at the center of the polyhedron, atoms of\n"
"type name2 are at the vertices and length is the maximum distance to be included.\n"
"\n"
"\n"
" rem            text\n"
" REM            text\n"
"\n"
"Any line preceded by this command is ignored.\n"
"\n"
"\n"
" shell         name length1 length2 color\n"
"\n"
"defines a polyhedral hull, where name is the name of an atom at the center of a polyhedron and\n"
"length1 and length2 are the mimimum and maximum lengths of distances to atoms that are to be\n"
"considered as the vertices of the polyhedron. The polyhedra can be of any desired complexity,\n"
"and can be stacked as desired.\n"
"\n"
"\n"
" slab a b c alpha beta gamma xoff yoff zoff xrot yrot zrot flag\n"
"\n"
"defines a (possibly oblique) cutout box of the specified axis lengths and\n"
"angles that is offset by xoff,yoff zoff from the origin of the structure\n"
"and rotated at angles xrot yrot zrot relative to it. If flag is set to 1,\n"
"any part of the structure outside the box is deleted. If flag is 2, the\n"
"outline of the box is overlaid on the unchanged image to allow accurate\n"
"placement of the cutout box.\n" 
"\n"
"\n"
" spgp    symbol\n"
" spgr    symbol\n"
" sgrp    symbol\n"
"\n"
"Space Group name consisting of the Bravais lattice symbol (must be upper case) followed by a space,\n"
"the elements parallel to the first axis followed by a space, etc. Examples are I 41/a m d, P 21/n, I a 3 d,\n"
"P b n m, etc. The generators will always select the origin choice with a center of symmetry at the\n"
"origin. Furthermore, all monoclinic cells will have the unique axis parallel to the b axis, unless the\n"
"full symbol is used, i.e. P 1 1 21/n describes a monoclinic cell with c as the unique axis. N.B.:\n"
"Rhombohedral space groups must be represented in the hexagonal form.\n"
"\n"
"\n"
" sphere         name radius color\n"
" sphere         name number radius color\n"
"\n"
"where name is a one- or two-character symbol of the atom type, radius is the radius of the sphere in\n"
"Angstrom, and color is the color of the sphere to be drawn. If the first form is used, all atoms with\n"
"that name will be drawn. The second restricts the command to that name and number only\n"
"\n"
"\n"
" title, titl    text\n"
"\n"
"General description of the structure - this line may appear anywhere in the file, but is generally first.\n"
"\n"
"\n"
" uij, Uij       name number u color\n"
"\n"
"defines the anisotropic thermal coefficients for an atom and color of the ellipsoid. The name and\n"
"number should correspond to the atom input described above.\n"
"\n"
"\n"
" vectors     posx  posy  posz\n"
"\n"
"turns on the orientation vector triple at a corner of the diagram. The optional parameters pos[xyz]\n"
"specify the location of the origin. If not given, the program will guess at a location\n"
"\n"
"\n"
" vrml1\n"
"\n"
"causes the output VRML file to have the VRML1 syntax (as opposed to the newer VRML97 standard)\n"
"\n"
"\n"
" xyzoff         u1  u2 u3\n"
"\n"
"causes all atom coordinates to be shifted by -u.  This command is used whenever the origin defined for\n"
"a structure does not conform to the standard origin selected by the space-group generator.\n"
"\n"
"\n"
" end\n"
" END\n"
"\n"
"The last line of a file that is read. Any information past this point will be ignored.\n"
"\n"
"\n"
"Sample input file\n"
"\n"
"title Buckyball with balls and sticks\n"
"cell 14.16 14.16  14.16\n"
"pack -.3 .3 -.3 .3 -.3 .3\n"
"spgp F m 3\n"
"sphere c 0.4 Red filter 0.3\n"
"bond c c 0.1 1.2 1.5 Gray30\n"
"atom c 1 0.04908 0.00000 0.24510\n"
"atom c 2 0.10028 0.08284 0.21346\n"
"atom c 3 0.18313 0.05120 0.16226\n"
"origin 0 0 0\n"
"magnification 0.7\n"
"view -18 0 0\n"
"end\n"};

static int zero = 0;

    if (helpwindow) {
        helpwindow->show();
        return;
    }
    helpwindow = new Fl_Window(50, 50, 680, 550, "Input File Commands");
    helpwindow->begin();
    helpwindow->callback((Fl_Callback*)View_Help_Close_cb, &zero);
    Fl_Text_Display * display = new Fl_Text_Display(0, 0, 680, 510);
    display->textsize(13);
    display->textfont(FL_HELVETICA_BOLD);
    Fl_Text_Buffer * m = new Fl_Text_Buffer();
    display->buffer(m);
    m->text(helptext);
    Fl_Button* o = new Fl_Button(300, 515, 80, 30, "Close");
    o->callback((Fl_Callback*)View_Help_Close_cb, &zero);
#if !defined (WIN32) && !defined (__APPLE__)
    helpwindow->icon((char *)icon);
#endif
    helpwindow->end();
    helpwindow->show();
}

Fl_Double_Window* make_window() {
  Fl_Double_Window* w;
  { Fl_Double_Window* o = new Fl_Double_Window(100, 50, 518, 429, "DRAWxtl-Shell V4.2 Control Program");
    w = o;
    o->color(FL_WHITE);
    o->selection_color(FL_WHITE);
    o->labelcolor(FL_RED);
    o->align(FL_ALIGN_CENTER);
    { Fl_Menu_Bar* o = new Fl_Menu_Bar(0, 0, 518, 25);
      o->menu(menu_File);
      o->color(FL_WHITE);
    }
    {
      Fl_Button * o = new Fl_Button( 468, 0, 50, 25, "Exit");
      o->callback((Fl_Callback*)Exit_cb);
      o->color(FL_WHITE);
    }
    { Fl_Text_Display* o = new Fl_Text_Display(70, 122, 391, 0, "DRAWxtl V4.2");
      o->labelfont(9);
      o->labelsize(52);
      o->labelcolor((Fl_Color)1);
      o->textcolor(1);
    }
    { Fl_Text_Display* o = new Fl_Text_Display(0, 146, 526, 0, "Crystal Structure Drawing Program");
      o->labelfont(9);
      o->labelsize(24);
      o->labelcolor((Fl_Color)69);
    }
    { Fl_Text_Display* o = new Fl_Text_Display(0, 179, 530, 0, "L.W. Finger, M. Kroeker and B. Toby");
      o->labelfont(11);
      o->labelsize(20);
      o->labelcolor((Fl_Color)4);
    }
    { Fl_Text_Display* o = new Fl_Text_Display(0, 207, 530, 0, "Modified 08/17/05");
      o->labelfont(8);
      o->labelsize(18);
      o->labelcolor((Fl_Color)1);
    }
    { Fl_Output* o = Current_Directory = new Fl_Output(187, 255, 223, 25, "Current Directory:");
      o->box(FL_FLAT_BOX);
      o->color(FL_WHITE);
      o->labelfont(9);
      o->labelsize(13);
      o->textcolor((Fl_Color)1);
      o->textfont(FL_HELVETICA_BOLD);
      o->textsize(13);
    }
    { Fl_Output* o = Current_Structure_File = new Fl_Output(187, 278, 223, 25, "Current Structure File:");
      o->box(FL_FLAT_BOX);
      o->color(FL_WHITE);
      o->labelfont(9);
      o->labelsize(13);
      o->textcolor((Fl_Color)1);
      o->textfont(FL_HELVETICA_BOLD);
      o->textsize(13);
    }
    { Fl_Output* o = Command_Line_Options = new Fl_Output(187, 300, 223, 25, "Command Line Options:");
      o->box(FL_FLAT_BOX);
      o->color(FL_WHITE);
      o->labelfont(9);
      o->labelsize(13);
      o->textcolor((Fl_Color)1);
      o->textfont(FL_HELVETICA_BOLD);
      o->textsize(13);
      o->value(DRAWxtl_Option);
    }
    { Fl_Output* o = POV_Options = new Fl_Output(187, 322, 223, 25, "POV Options:");
      o->box(FL_FLAT_BOX);
      o->color(FL_WHITE);
      o->labelfont(9);
      o->labelsize(13);
      o->textcolor((Fl_Color)1);
      o->textfont(FL_HELVETICA_BOLD);
      o->textsize(13);
    }
    o->end();
  }
  return w;
}

/******************************************************************************/
/******************************************************************************/

void Options_Close_cb(void)
{
    optionswindow->hide();
}

/******************************************************************************/
/******************************************************************************/

void Options_Save_cb(void)
{
    optionswindow->hide();
    Command_Line_Options->value(drawxtl_options->value());
    POV_Options->value(pov_options->value());
    strcpy(POV_Option,pov_options->value());
    strcpy(DRAWxtl_Option,drawxtl_options->value());
    if (!strstr(drawxtl_options->value(),"-a")) {
        autolabel = 0;
    } else {
        autolabel = 1;
    }
    WriteConfig();
}

/******************************************************************************/
/******************************************************************************/

void Run_DRAWxtl_cb(void)
{
    char cmd[256];

    if (!strlen(Cur_File)) {            // Make sure rendering enabled
        Error_Box("A Structure File must be selected first.");
        return;
    }
#ifdef WIN32
    strcpy(cmd,"\"");                         // build the command string
    strcat(cmd,DRAWxtl_Path);                 //   for Windows
    strcat(cmd,"\"");
#else
    strcpy(cmd,DRAWxtl_Path);                 // build command string
    strcat(cmd," ");                          //    for Linux
#endif
    strcat(cmd," ");
    strcat(cmd,DRAWxtl_Option);
    strcat(cmd," ");
    strcat(cmd,Cur_Dir);
    strcat(cmd,Cur_File);
    system(cmd);
}

/******************************************************************************/
/******************************************************************************/

void  SelectDataFile_cb(void)
{
// Callback routine to select the data file
    FILE * inp;
    int i;
#ifdef WIN32
    char drive[_MAX_DRIVE];
    char dir[_MAX_DIR];
    char fname[_MAX_FNAME];
    char ext[_MAX_EXT];
    const char * newfile = flu_file_chooser("Select Data/Experiment File",
        "*.str", Cur_File);
    if(newfile){
        _splitpath( newfile, drive, dir, fname, ext );  //Windows code
        strcpy(Cur_Dir,drive);                // Drive letter
        strcat(Cur_Dir,dir);                  //   and directory
        strcpy(Cur_File,fname);               // copy file name
        strcat(Cur_File,ext);                 // and add extension
#else
    int k = 0;
    char * newfile = fl_file_chooser("Select Data/Experiment File", "*.str", Cur_File);
    if(newfile){
        strcpy(Cur_File,newfile);
        strcpy(Cur_Dir,newfile);
        for (i=strlen(Cur_Dir);i>0;--i) {     // Find final / in file name
            if (Cur_Dir[i-1] == '/') {
                Cur_Dir[i] = 0;
                break;
            }
        }
        for (i=strlen(Cur_Dir);newfile[i] != 0;i++) {
            Cur_File[k++] = newfile[i];       // copy file name to Cur_File
        }
        Cur_File[k] = 0;
#endif

        strcpy(Cur_Root,Cur_File);
        for (i=strlen(Cur_Root); i>0; --i) {
            if (Cur_Root[i] == '.') {
                Cur_Root[i] = '\0';
                break;
            }
        }
        chdir(Cur_Dir);                              // switch to new directory
        WriteConfig();                               // update configuration file
        Current_Directory->value(Cur_File);          // update main screen widgets
        Current_Structure_File->value(Cur_Dir);
        if(!(inp=fopen(Cur_File,"r"))) {
            char string[256];
            sprintf(string,"The file you selected ('%s') cannot be read\n"
                "Do you wish to continue?",Cur_File);
            if(fl_ask(string)) {
                inp = fopen(Cur_File,"w");
                fclose(inp);
                Edit_Data_cb();
            }
        } else {
            fclose(inp);
        }
        Fl::redraw();
    }
}

/******************************************************************************/
/******************************************************************************/

void Set_Options_cb(void)
{
    int y = 40;

    if (!optionswindow) {
        optionswindow = new Fl_Window(100,100,400,200,"Set Options");
#if !defined (WIN32) && !defined (__APPLE__)
        optionswindow->icon((char *)icon);
#endif
        optionswindow->callback((Fl_Callback*)Options_Close_cb);
        optionswindow->color(FL_WHITE);
        drawxtl_options = new Fl_Input(130, y, 240, 25, "DRAWxtl Options:");
        drawxtl_options->value(Command_Line_Options->value());
        y += 40;
        pov_options = new Fl_Input(130, y, 240, 25, "POV Options");
        pov_options->value(POV_Options->value());
        y += 40;
        Fl_Button * b1 = new Fl_Button( 120, y, 70, 25, "Close");
        b1->tooltip("Close this window and discard all changes.");
        b1->callback((Fl_Callback*)Options_Close_cb);
        Fl_Button * b2 = new Fl_Button( 210, y, 70, 25, "Save");
        b2->callback((Fl_Callback*)Options_Save_cb);
        optionswindow->end();
    }
    optionswindow->show();
}

/******************************************************************************/
/******************************************************************************/

void trim_string(char string[], int len)
{
// trim string - remove trailing ^J (Windows) and spaces. 'len' is maximum length
    int i;
    string[len-1] = 0;
    if (strlen(string) == 0) {
        strcpy(string,"White");
        return;
    }
    for (i=0;i<(int)strlen(string);i++) {
        if (string[i] < ' ') {
            string[i] = 0;
            break;
        }
    }
    for (i=strlen(string);i > 0; --i) {
        if (string[i-1] <= ' ') {
            string[i-1] = 0;
        } else
            break;
    }
}

/******************************************************************************/
/******************************************************************************/

void View_Help_Close_cb(Fl_Window *, int * arg)    // callback to destruct help window
{
// callback to close the STR help screens
    switch (* arg) {
    case 0:
        helpwindow->hide();
        break;
    case 1:
        helpwindow1->hide();
        break;
    case 2:
        helpwindow2->hide();
        break;
    case 3:
        helpwindow3->hide();
        break;
    case 4:
        helpwindow4->hide();
        break;
    }
}

/******************************************************************************/
/******************************************************************************/

void View_Listing_cb(void)      // callback to list the '.out' file
{
    static int two = 2;
    if (!strlen(Cur_File)) {         // Make sure rendering enabled
        Error_Box("A Structure File must be selected first.");
        return;
    }
// callback to view listing file
    if (!listwindow2) {
        listwindow2 = new Fl_Window(50, 50, 680, 550, "Listing View");
        listwindow2->begin();
        listwindow2->callback((Fl_Callback*)View_List_Close_cb, &two);
        Fl_Text_Display * display = new Fl_Text_Display(0, 0, 680, 510);
        display->textfont(FL_COURIER);
        textbuf = new Fl_Text_Buffer;
        display->buffer(textbuf);
        Fl_Button* o = new Fl_Button(300, 515, 80, 30, "Close");
        o->tooltip("Close this window and discard all changes.");
        o->callback((Fl_Callback*)View_List_Close_cb, &two);
        listwindow2->end();
#if !defined (WIN32) && !defined (__APPLE__)
        listwindow2->icon((char *)icon);
#endif
    }
    textbuf->loadfile(Cur_Listing);
    listwindow2->show();
}

/******************************************************************************/
/******************************************************************************/

void View_List_Close_cb(Fl_Window *, int * arg)    // callback to hide listing windows
{
    switch (*arg) {
    case 1:
        listwindow1->hide();
        break;
    case 2:
        listwindow2->hide();
        break;
    }
}

/******************************************************************************/
/******************************************************************************/

void View_POV_cb(void)
{
// Callback routine to view the POV file
    char cmd[512];
    if (!strlen(Cur_File)) {         // Make sure rendering enabled
        Error_Box("A Structure File must be selected first.");
        return;
    }
#ifdef WIN32
    strcpy(cmd,"\"");                         // build the command string
    strcat(cmd,POV_Path);                     //   for Windows
    strcat(cmd,"\"");
    strcat(cmd," ");
    strcat(cmd,POV_Option);
    strcat(cmd," ");
    strcat(cmd," +I");
#else
    strcpy(cmd,POV_Path);                     // build command string
    strcat(cmd," ");                          //    for Linux
    strcat(cmd,POV_Option);
    strcat(cmd," ");
#endif
    strcat(cmd,Cur_Root);
    strcat(cmd,".pov");
    system(cmd);                              // call the POV program
}

/******************************************************************************/
/******************************************************************************/

void View_VRML_cb(void)
{
    char cmd[256];

    if (!strlen(Cur_File)) {            // Make sure rendering enabled
        Error_Box("A Structure File must be selected first.");
        return;
    }
#ifdef WIN32
    strcpy(cmd,"\"");                         // build the command string
    strcat(cmd,VRML_Path);                    //   for Windows
    strcat(cmd,"\"");
#else
    strcpy(cmd,VRML_Path);                     // build command string
    strcat(cmd," ");                          //    for Linux
#endif
    if (!strstr(VRML_Use_Browser,"no"))
        strcat(cmd," file://");
    else
        strcat(cmd," ");
    strcat(cmd,Cur_Dir);
    strcat(cmd,Cur_Root);
    strcat(cmd,".wrl");
    system(cmd);                              // call the VRML program
}

/******************************************************************************/
/******************************************************************************/

void WriteConfig(void)
{
// write updated configuration file
    FILE *fname;
    char filename[256];
    
#ifdef WIN32
    char profile[100];
    OSVERSIONINFOEX osvi;
	
    memset(&osvi, 0, sizeof(OSVERSIONINFOEX));
    osvi.dwOSVersionInfoSize = sizeof(OSVERSIONINFOEX);
    GetVersionEx ((OSVERSIONINFO *)&osvi);
	if (osvi.dwMajorVersion == 4)
		strcpy(profile,"c:");
	else
	    strcpy(profile,getenv("USERPROFILE"));
    strcpy(filename,profile);
    strcat(filename,"\\");
    strcat(filename,Configure_file);
#else
    fl_filename_expand(filename,Configure_file);
#endif
    fname = fopen(filename,"w");
    if (!fname) {
        return;
    }
    fprintf(fname, "%s\n%s\n",DRAWxtl_Path,POV_Path);
    fprintf(fname, "%s\n%s\n",VRML_Path,VRML_Use_Browser);
    fprintf(fname, "%s\n%s\n",DRAWxtl_Option,POV_Option);
    fprintf(fname, "%s\n%s\n",Cur_File,Cur_Dir);
    fprintf(fname, "%s\n",POV_Include);
    fprintf(fname, "%s\n",LoadOnStartup);
    fprintf(fname, "%s\n",DefaultFinish);
    fprintf(fname, "%s\n",GUIExePath);
    if (autolabel==0)
      fprintf(fname,"noautolabel\n");
    else
      fprintf(fname,"autolabel\n");
    fclose(fname);
}
