/* draw_ext.h - external definitions for DRAWxtl */

extern CRYSTAL *drvui;

/* global variables for atom lists */

extern int natom;                  /* number of different types of atoms */
extern int ncell;                  /* number of atoms in asymmetric unit */
extern float *xypos;               /* master list fractional coordinates */
extern float *o_vert;              /* crystal coordinates of saved vertices */
extern float *s_vert;              /* saved coordinates of vertices */
extern int *vert_sym_no;           /* saved symmetry operator number */
extern int *poly_list;             /* storage for polygon corner pointers */
extern int nvert;                  /* number of vertices in list */
extern int NvertM;                 /* number of vertices in master atom list */

/* global variables for spheres and ellipsoids */

extern int n_ellips;				 /* no. of aniso. atoms in asymmetric unit */
extern int do_ellipsoids;			 /* non-zero if ellipsoids to be drawn */
extern int El_Cutout;				 /* True if ellipsoids to have an octant removed */
extern float Sphere_Mult;			 /* Multiplier for Sphere Size */
extern float SpMult;				 /* initialization for Sphere_Mult */
extern float Ellipsoid_Prob;			 /* Fraction of electron dens enclosed by ellipsoid */
extern float Ellipsoid_Scale;			 /* Scaling needed to get probability */
extern char Ellipaxis_color[40];		 /* color of principal axes */
extern float Ellipaxis_width;			 /* linewidth of principal axes */

/* global variables for polyhedra, planes and bonds */

extern int draw_list;				 /* number of items in polygon draw list */
extern int *vertex_list;			 /* storage for vertices about a cation */
extern int numb_list;				 /* number in polygon list */
extern int domolcomp;				 /* non-zero if molecule completion requested */

/* global variables that control size and orientation of output object */

extern float DepthCue;				 /* scale for Z-dependent edge radius */
extern float boxlim[3];				 /* half limits of plotting box */
extern float origin[3];				 /* position of plotting origin */
extern double G_Rot[3][3];			 /* Grand rotation matrix - transforms from Cartesian to picture */
extern float POV_Max[3];			 /* Maximum limits of POV after rotation */
extern float POV_Min[3];			 /* Minimum limits of POV after rotation */
extern int boxflag;				 /* Non-zero if 'bounds' command given */
extern int packflag;				 /* Non-zero if 'pack' command given */
extern int clipflag;				 /* Non-zero if 'clip' command given */
extern int docell;				 /* non-zero if unit-cell edges to be drawn */
extern int Display_axes;			 /* non-zero if axial triple to be drawn */
extern int Color_Warning;			 /* Warning flag for non-standard color */
extern float Magnification;			 /* Magnification factor for image */
extern float Scale;				 /* Scale of diagram */
extern float offset[3];				 /* offset for vector triple */
extern float Text_Size;				 /* Size to make axis label text */
extern int Options;				 /* Place to save command-line options flags */
extern char col_cell_v[40];			 /* color and properties of framebars */
extern char col_edge_v[40];			 /* color and properties of polyhedron edges */
extern char col_cell_p[40];			 /* color and properties of framebars */
extern char col_edge_p[40];			 /* color and properties of polyhedron edges */
extern float rad_cell;				 /* radius of unit cell framebars */
extern int edges;                    /* draw thin black lines around the edges of polyhedra */
extern float rad_edge;				 /* radius of cylinders on polyhedron edges */
extern float xrot, yrot, zrot;       /* view rotation angles */
extern int Unit_Cell;				 /* non-zero if unit cell to be drawn */
extern int no_comment;				 /* non-zero to inhibit comment lines in VRML */
extern int M_cameras;				 /* non-zero to inhibit multiple cameras */
extern int Vrml2;                    /* non-zero to generate VRML97 (VRML2) output */
extern float printdist;				 /* Distance limit for tabulated output */
extern int clipflag;
extern int Labels;                   /* True if labels should be output */
extern int gl_view;	                 /* if true, generate openGL view */

#ifdef OPENGL
extern GLuint crystalDL;
extern int gl_ortho;
extern GLdouble modelMatrix[16];
extern GLdouble projMatrix[16];
extern GLint viewport[4];
#endif

extern int slabmode;
extern float slabx1,slaby1,slabz1,slabx2,slaby2,slabz2;
extern float slabx3,slaby3,slabz3,slabx4,slaby4,slabz4;

/* openGL cursor stuff */
extern int cur_show;			         /* true = show cursor; 0=no */
extern float cur_cen[3];			 /* location of cursor (fractional coordinates) */
extern int cur_atom[4];                 /* sequence numbers of last four atoms under cursor */
extern char cur_name[4][10];            /* name of atom under cursor */
extern float dist12;
extern float dist23;
extern float dist34;
extern float ang123;
extern float ang234;
extern float torsion_ang;
extern float cur_step;                           /* amount to jump cursor */

/* Fourier map stuff */
#include "drawmap.h"
extern int ReadFourMap;                         /* true if a Fourier map has been read */
extern float * FourierPt;                       /* pointer to Fourier map contents */
extern float map_a, map_b, map_c, map_alpha, map_beta, map_gamma; /* cell dimensions for map */
extern int  mapstep_a, mapstep_b, mapstep_c;     /* map steps across unit cell */
extern float xMin, xMax, yMin, yMax, zMin, zMax; /* bounding box for map */
extern int   numOfFourierContours;
extern int  FourierContourSolid[MAX_MAP_CONTOUR];
extern float FourierContourLevel[MAX_MAP_CONTOUR];
extern char FourierContourColor[MAX_MAP_CONTOUR][40];
extern char FourierFileName[100];
extern int FourierMapType;
extern MAP_INFO Map_Info;

#ifdef OPENGL
extern GLuint crystalDL;
#endif

extern int omit1[1000],omit2[1000],omit3[1000];
extern int nomits;
extern int gl_width;
extern int gl_height;
extern float xrot_i, yrot_i, zrot_i;
extern float gl_size;
#ifdef OPENGL
extern int gl_oldx, gl_oldy, gl_oldz;	/* cursor position at beginning of rotation */
extern int gl_xymotion, gl_zmotion;
extern float gl_pos_x, gl_pos_y, gl_pos_z;
extern int gl_ortho;
extern float gl_offset;
#endif

extern float xrot_i, yrot_i, zrot_i;
extern QUAT Rotq;
