#include "drawxtl.h"
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <stdlib.h>
#include <time.h>
#include "draw_ext.h"


/* *************************************************************************************** */
/* Generate Marching Cubes triangles from a map with contour minValue, Color is POV color, */
/*      Solid is false for mesh                                                            */
/* *************************************************************************************** */

void generate_map (float minValue, int Solid, char * Color)
{
  int numOfTriangles;
  float step[3];
  float Red, Green, Blue;
  int nxMin, nxMax, nyMin, nyMax, nzMin, nzMax; /* max and min steps along a, b & c */
  int snx, sny, snz; /* number of steps along a, b & c */
  unsigned int sny_snz; /* sny x snz */
  int i, j, k, ijk, ii, jj;
  unsigned int ni, nj, si, sj, sk, sni, snj, sind;
  mp4Vector * PointsList;
  float fvert[3], cvert[3];
  TRIANGLE *pTriangles;
  char string[40];
  float d1, d2, d3;
  int valid_tri;

  strcpy(string,Color);
  check_color(string);
  (void) sscanf(string, "%f %f %f", &Red, &Green, &Blue);

  step[0]= 1.0f / (float)mapstep_a;
  step[1]= 1.0f / (float)mapstep_b;
  step[2]= 1.0f / (float)mapstep_c;
  nxMin = (int)(max(xMin,drvui->cryst_lim[drvui->frame_no][0]) * mapstep_a);
  nxMax = (int)(min(xMax,drvui->cryst_lim[drvui->frame_no][3]) * mapstep_a);
  nyMin = (int)(max(yMin,drvui->cryst_lim[drvui->frame_no][1]) * mapstep_b);
  nyMax = (int)(min(yMax,drvui->cryst_lim[drvui->frame_no][4]) * mapstep_b);
  nzMin = (int)(max(zMin,drvui->cryst_lim[drvui->frame_no][2]) * mapstep_c);
  nzMax = (int)(min(zMax,drvui->cryst_lim[drvui->frame_no][5]) * mapstep_c);
  snx = nxMax-nxMin+1; sny = nyMax-nyMin+1; snz = nzMax-nzMin+1;
  sny_snz = sny*snz;
  
 /* create a new set of Fourier points */
  PointsList = (mp4Vector *) malloc(sizeof(mp4Vector) * snx*sny*snz);
  if (PointsList == NULL)
      fprintf(stderr,"Error allocating space for PointsList vectors");
  for(i=nxMin, si=0; i <= nxMax; i++, si++) {
    ni = ((mapstep_a + i % mapstep_a) % mapstep_a);    /* this will 'wrap' around any value (negative or positive) */
    sni = si*sny_snz;
    for (j=nyMin, sj=0; j <= nyMax; j++, sj++) {
      nj = ((mapstep_b + j % mapstep_b) % mapstep_b);
      snj = sj*snz;
      for (k=nzMin, sk=0; k <= nzMax; k++, sk++) {
        sind = sni + snj + sk;
        fvert[0] = step[0]*i; fvert[1] = step[1]*j; fvert[2] = step[2]*k;
	/* fprintf(stderr," grid fractional: %f %f %f\n",x,y,z); */
   /* convert Fractional to Orthonormal Coords */
        for (ii = 0; ii <= 2; ++ii) {			 /* convert vertex coordinates to Cartesian */
          cvert[ii] = 0.0f;
          for (jj = 0; jj <= 2; ++jj)
            cvert[ii] += (float) drvui->b_mat[ii][jj] * (fvert[jj] - origin[jj]);
        } 
        PointsList[sind].x = cvert[0];
        PointsList[sind].y = cvert[1];
        PointsList[sind].z = cvert[2];
/* 	fprintf(stderr," grid ortho: %f %f %f\n",PointsList[sind].x, PointsList[sind].y, PointsList[sind].z);  */
        ijk =  ni*mapstep_b*mapstep_c + nj*mapstep_c + ((mapstep_c+ k % mapstep_c)% mapstep_c);
        PointsList[sind].val = FourierPt[ijk];
      }
    }
  }

  /* create contours from grid of points with matching cubes */
  pTriangles= MC_c(snx-1, sny-1, snz-1, step[0], step[1], step[2], minValue, PointsList, &numOfTriangles);
  free (PointsList); 

/*   for(i=0; i < numOfTriangles; i++) { */
/*     for(j=0; j < 3; j++) { */
/*       fprintf(stderr," triangle %d fractional: %f %f %f\n",i, */
/* 	      pTriangles[i].p[j].x, pTriangles[i].p[j].y, pTriangles[i].p[j].z); */
/*     } */
/*   } */


#ifdef OPENGL
  if (gl_view) {
    glPushMatrix ();
    if (!Solid) 
      glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
    else
      glMaterialf( GL_FRONT, GL_SHININESS, 0.0); /* disable shinyness */
    glColor3f (Red, Green, Blue);
    glBegin(GL_TRIANGLES);
  }
#endif
  for(i=0; i < numOfTriangles; i++) {
    if (minValue>0) {

      if (!Vrml2) {
        fprintf(drvui->fpoutv," Separator {\n");
        fprintf(drvui->fpoutv," Material {  diffuseColor %s }\n",string);
        fprintf(drvui->fpoutv,"Coordinate3 { point [\n");
      } else {
        fprintf(drvui->fpoutv," Shape {\n");
        if (Solid) {
          fprintf(drvui->fpoutv,"  appearance Appearance {\n");
          fprintf(drvui->fpoutv,"   material Material {diffuseColor %s}\n",string);
          fprintf(drvui->fpoutv,"  }\n  geometry IndexedFaceSet { coord Coordinate{ point [\n");
        } else
          fprintf(drvui->fpoutv,"  geometry IndexedLineSet { coord Coordinate{ point [\n");
      }

      d1 =    (pTriangles[i].p[0].x - pTriangles[i].p[1].x) *
              (pTriangles[i].p[0].x - pTriangles[i].p[1].x) +
              (pTriangles[i].p[0].y - pTriangles[i].p[1].y) *
              (pTriangles[i].p[0].y - pTriangles[i].p[1].y) +
              (pTriangles[i].p[0].z - pTriangles[i].p[1].z) *
              (pTriangles[i].p[0].z - pTriangles[i].p[1].z);
      d2 =    (pTriangles[i].p[0].x - pTriangles[i].p[1].x) *
              (pTriangles[i].p[2].x - pTriangles[i].p[1].x) +
              (pTriangles[i].p[2].y - pTriangles[i].p[1].y) *
              (pTriangles[i].p[2].y - pTriangles[i].p[1].y) +
              (pTriangles[i].p[2].z - pTriangles[i].p[1].z) *
              (pTriangles[i].p[2].z - pTriangles[i].p[1].z) *
              (pTriangles[i].p[0].x - pTriangles[i].p[0].x);
      d3 =    (pTriangles[i].p[2].x - pTriangles[i].p[0].x) +
              (pTriangles[i].p[2].y - pTriangles[i].p[0].y) *
              (pTriangles[i].p[2].y - pTriangles[i].p[0].y) +
              (pTriangles[i].p[2].z - pTriangles[i].p[0].z) *
              (pTriangles[i].p[2].z - pTriangles[i].p[0].z);
	  valid_tri = (d1 > 0.00001f) && (d2 > 0.0001f) && (d3 > 0.0001f);
      if (!Solid){
        if (d1 > 0.00001f) {
            fprintf (drvui->fpoutp," cylinder { <%f,%f,%f>,<%f,%f,%f>,%f\n",
              pTriangles[i].p[0].x,pTriangles[i].p[0].y,pTriangles[i].p[0].z,        
              pTriangles[i].p[1].x,pTriangles[i].p[1].y,pTriangles[i].p[1].z,
              0.0001*Scale);
            fprintf (drvui->fpoutp,"  texture{pigment{color %s  }}\n }\n", Color);
        }
        if (d2 > 0.00001f) {
            fprintf (drvui->fpoutp," cylinder { <%f,%f,%f>,<%f,%f,%f>,%f\n",
              pTriangles[i].p[1].x,pTriangles[i].p[1].y,pTriangles[i].p[1].z,        
              pTriangles[i].p[2].x,pTriangles[i].p[2].y,pTriangles[i].p[2].z,
              0.0001*Scale);
            fprintf (drvui->fpoutp,"  texture{pigment{color %s  }}\n }\n", Color);
        }
        if (d3 > 0.00001f) {
            fprintf (drvui->fpoutp," cylinder { <%f,%f,%f>,<%f,%f,%f>,%f\n",
              pTriangles[i].p[2].x,pTriangles[i].p[2].y,pTriangles[i].p[2].z,        
              pTriangles[i].p[0].x,pTriangles[i].p[0].y,pTriangles[i].p[0].z,
              0.0001*Scale);
            fprintf (drvui->fpoutp,"  texture{pigment{color %s  }}\n }\n", Color);
        }
      }
      if (Solid) fprintf(drvui->fpoutp,"triangle {\n");
      for(j=0; j < 3; j++) {
#ifdef OPENGL
        if (Solid) 
          glNormal3f(pTriangles[i].norm[j].x, pTriangles[i].norm[j].y, pTriangles[i].norm[j].z);
        glVertex3f(pTriangles[i].p[j].x, pTriangles[i].p[j].y, pTriangles[i].p[j].z);
#endif
        if (Solid) 
          fprintf(drvui->fpoutp,"<%8.5f, %8.5f, %8.5f>",pTriangles[i].p[j].x, pTriangles[i].p[j].y, pTriangles[i].p[j].z);
        fprintf(drvui->fpoutv," %5.3f %5.3f %5.3f",pTriangles[i].p[j].x, pTriangles[i].p[j].y, pTriangles[i].p[j].z);
        if (j<2) {
          if (Solid) 
            fprintf(drvui->fpoutp,",");
          fprintf(drvui->fpoutv,",\n");
        } else {
          if (Solid) 
              fprintf(drvui->fpoutp,"\n texture{pigment{color %s }}\n }\n", Color);
          fprintf(drvui->fpoutv,"]\n");
        }/* for (j.... */
      } 
      if (Vrml2) {
        if (Solid) {
            fprintf(drvui->fpoutv,"  }\n  coordIndex [ 0,1,2,-1]\n solid FALSE\n convex TRUE\n creaseAngle 1.57075\n }\n }\n");
        } else {
            fprintf(drvui->fpoutv,"  }\n  coordIndex [ 0,1,2,-1]\n color Color { color [%s]"
               "}\n colorIndex [0]\n colorPerVertex FALSE\n }\n }\n",string);
        }
      } else {
        if (Solid) {
            fprintf(drvui->fpoutv,"}\n IndexedFaceSet { coordIndex [0,1,2,-1] }\n}\n");
        } else {
            fprintf(drvui->fpoutv,"}\n IndexedLineSet { coordIndex [0,1,2,-1] }\n}\n");
        }
      }
    } else {           /* if (minValue > 0) */
      if (!Vrml2) {
        fprintf(drvui->fpoutv," Separator {\n");
        fprintf(drvui->fpoutv," Material {  diffuseColor %s }\n",string);
        fprintf(drvui->fpoutv,"Coordinate3 { point [\n");
      } else {
        fprintf(drvui->fpoutv," Shape {\n");
        if (Solid) {
          fprintf(drvui->fpoutv,"  appearance Appearance {\n");
          fprintf(drvui->fpoutv,"   material Material {diffuseColor %s}\n",string);
          fprintf(drvui->fpoutv,"  }\n  geometry IndexedFaceSet { coord Coordinate{ point [\n");
        } else
          fprintf(drvui->fpoutv,"  geometry IndexedLineSet { coord Coordinate{ point [\n");
      }

      d1 =    (pTriangles[i].p[0].x - pTriangles[i].p[1].x) *
              (pTriangles[i].p[0].x - pTriangles[i].p[1].x) +
              (pTriangles[i].p[0].y - pTriangles[i].p[1].y) *
              (pTriangles[i].p[0].y - pTriangles[i].p[1].y) +
              (pTriangles[i].p[0].z - pTriangles[i].p[1].z) *
              (pTriangles[i].p[0].z - pTriangles[i].p[1].z);
      d2 =    (pTriangles[i].p[0].x - pTriangles[i].p[1].x) *
              (pTriangles[i].p[2].x - pTriangles[i].p[1].x) +
              (pTriangles[i].p[2].y - pTriangles[i].p[1].y) *
              (pTriangles[i].p[2].y - pTriangles[i].p[1].y) +
              (pTriangles[i].p[2].z - pTriangles[i].p[1].z) *
              (pTriangles[i].p[2].z - pTriangles[i].p[1].z) *
              (pTriangles[i].p[0].x - pTriangles[i].p[0].x);
      d3 =    (pTriangles[i].p[2].x - pTriangles[i].p[0].x) +
              (pTriangles[i].p[2].y - pTriangles[i].p[0].y) *
              (pTriangles[i].p[2].y - pTriangles[i].p[0].y) +
              (pTriangles[i].p[2].z - pTriangles[i].p[0].z) *
              (pTriangles[i].p[2].z - pTriangles[i].p[0].z);
      if (!Solid) {
        if (d1 > 0.00001f) {           /* skip zero length cylinder */
            fprintf (drvui->fpoutp," cylinder { <%f,%f,%f>,<%f,%f,%f>,%f\n",
              pTriangles[i].p[0].x,pTriangles[i].p[0].y,pTriangles[i].p[0].z,        
              pTriangles[i].p[1].x,pTriangles[i].p[1].y,pTriangles[i].p[1].z,
              0.0001*Scale);
            fprintf (drvui->fpoutp,"  texture{pigment{color %s  }}\n }\n", Color);
        }
        if (d2 > 0.0001f) {           /* skip zero length cylinder */
            fprintf (drvui->fpoutp," cylinder { <%f,%f,%f>,<%f,%f,%f>,%f\n",
              pTriangles[i].p[1].x,pTriangles[i].p[1].y,pTriangles[i].p[1].z,        
              pTriangles[i].p[2].x,pTriangles[i].p[2].y,pTriangles[i].p[2].z,
              0.0001*Scale);
            fprintf (drvui->fpoutp,"  texture{pigment{color %s  }}\n }\n", Color);
        }
        if (d3 > 0.0001f) {           /* skip zero length cylinder */
            fprintf (drvui->fpoutp," cylinder { <%f,%f,%f>,<%f,%f,%f>,%f\n",
              pTriangles[i].p[2].x,pTriangles[i].p[2].y,pTriangles[i].p[2].z,        
              pTriangles[i].p[0].x,pTriangles[i].p[0].y,pTriangles[i].p[0].z,
              0.0001*Scale);
            fprintf (drvui->fpoutp,"  texture{pigment{color %s  }}\n }\n", Color);
        }
      }
      if (Solid) fprintf(drvui->fpoutp,"triangle {\n");
      for(j=2; j >=0; j--) {
#ifdef OPENGL
        if (gl_view) {
          if (Solid) 
            glNormal3f(pTriangles[i].norm[j].x, pTriangles[i].norm[j].y, pTriangles[i].norm[j].z);
          glVertex3f(pTriangles[i].p[j].x, pTriangles[i].p[j].y, pTriangles[i].p[j].z);
        }
#endif
        if (Solid) 
          fprintf(drvui->fpoutp,"<%8.5f, %8.5f, %8.5f>",pTriangles[i].p[j].x, pTriangles[i].p[j].y, pTriangles[i].p[j].z);
        fprintf(drvui->fpoutv," %5.3f %5.3f %5.3f",pTriangles[i].p[j].x, pTriangles[i].p[j].y, pTriangles[i].p[j].z);
        if (j>0) {
          if (Solid) fprintf(drvui->fpoutp,",");
          fprintf(drvui->fpoutv,",\n");
        } else {
          if (Solid)
            fprintf(drvui->fpoutp,"\n texture{pigment{color %s }}\n }\n", Color);
          fprintf(drvui->fpoutv,"]\n");
        }
      }
      if (Vrml2) {
        if (Solid) {
          fprintf(drvui->fpoutv,"}\n coordIndex [ 1,2,3,-1]\n solid FALSE\n convex TRUE\n creaseAngle 1.57075\n }\n }\n");
        } else {
          fprintf(drvui->fpoutv,"  }\n  coordIndex [ 0,1,2,-1]\n color Color { color [%s]"
             "}\n colorIndex [0]\n colorPerVertex FALSE\n }\n }\n",string);
        }
      } else {
        if (Solid) {
          fprintf(drvui->fpoutv,"}\n IndexedFaceSet { coordIndex [1,2,3,-1] }\n}\n");
        } else {
          fprintf(drvui->fpoutv,"}\n IndexedLineSet { coordIndex [1,2,3,-1] }\n}\n");
        }
      }
    }              /* if (minValue > 0) */
  }
#ifdef OPENGL
  if (gl_view) {
    glEnd();
    if (!Solid)
      glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
    glPopMatrix ();
  }
#endif
  MCfree_c (pTriangles); /* free the contour array */
}

/* interpolate rho on a 3-D grid */

float InterpolateMap (float x, float y, float z)
{ 
  /* for simplicity, use weighting appropriate for a cubic grid (should be better than no interpolation) */
  int ix,iy,iz; 
  int ix1,iy1,iz1; 
  float Xmin, Ymin, Zmin, Xmax, Ymax, Zmax, AvgRho;

  /* Compute the grid points on either size of x, then map those points onto the 3D array indices */
  ix=((int)(x*mapstep_a));
  Xmin = ((float) ix)/mapstep_a;
  Xmax = ((float) (ix+1))/mapstep_a;
  ix=((int)(x*mapstep_a))%mapstep_a;
  ix1= (ix + 1) % mapstep_a;

  /* ditto for y & z */
  iy=((int)(y*mapstep_b));
  Ymin = ((float) iy)/mapstep_b;
  Ymax = ((float) (iy+1))/mapstep_b;
  iy=((int)(y*mapstep_b))%mapstep_b;
  iy1= (iy + 1) % mapstep_b;
  iz=((int)(z*mapstep_c));
  Zmin = ((float) iz)/mapstep_c;
  Zmax = ((float) (iz+1))/mapstep_c;
  iz=((int)(z*mapstep_c))%mapstep_c;
  iz1= (iz + 1) % mapstep_c;

/*   printf(" x = %f, ix = %d, Xmin = %f, Xmax = %f\n",x,ix,Xmin,Xmax); */
/*   printf(" y = %f, iy = %d, Ymin = %f, Ymax = %f\n",y,iy,Ymin,Ymax); */
/*   printf(" z = %f, iz = %d, Zmin = %f, Zmax = %f\n",z,iz,Zmin,Zmax); */

  AvgRho = 0.;
  AvgRho += FourierPt[LookupMap(ix, iy, iz)  ]* (Xmax-x) * (Ymax-y) * (Zmax-z);
  AvgRho += FourierPt[LookupMap(ix1,iy, iz)  ]* (x-Xmin) * (Ymax-y) * (Zmax-z);
  AvgRho += FourierPt[LookupMap(ix1,iy1,iz)]* (x-Xmin) * (y-Ymin) * (Zmax-z);
  AvgRho += FourierPt[LookupMap(ix1,iy, iz1)]* (x-Xmin) * (Ymax-y) * (z-Zmin);
  AvgRho += FourierPt[LookupMap(ix, iy1,iz)  ]* (Xmax-x) * (y-Ymin) * (Zmax-z);
  AvgRho += FourierPt[LookupMap(ix, iy1,iz1)]* (Xmax-x) * (y-Ymin) * (z-Zmin);
  AvgRho += FourierPt[LookupMap(ix, iy, iz1)]* (Xmax-x) * (Ymax-y) * (z-Zmin);
  AvgRho += FourierPt[LookupMap(ix1,iy1,iz1)]* (x-Xmin) * (y-Ymin) * (z-Zmin);

  AvgRho /= (Xmax-Xmin) * (Ymax-Ymin) * (Zmax-Zmin);

/*   printf(" I[%3d,%3d,%3d] = %f, weight = %f\n",ix,iy,iz, */
/* 	 FourierPt[LookupMap(ix, iy, iz)  ], (Xmax-x) * (Ymax-y) * (Zmax-z) */
/* 	 / ((Xmax-Xmin) * (Ymax-Ymin) * (Zmax-Zmin))); */
/*   printf(" I[%3d,%3d,%3d] = %f, weight = %f\n",ix1,iy,iz, */
/* 	 FourierPt[LookupMap(ix1,iy, iz)  ], (x-Xmin) * (Ymax-y) * (Zmax-z) */
/* 	 / ((Xmax-Xmin) * (Ymax-Ymin) * (Zmax-Zmin))); */
/*   printf(" I[%3d,%3d,%3d] = %f, weight = %f\n",ix1,iy1,iz, */
/* 	 FourierPt[LookupMap(ix1,iy1,iz)], (x-Xmin) * (y-Ymin) * (Zmax-z) */
/* 	 / ((Xmax-Xmin) * (Ymax-Ymin) * (Zmax-Zmin))); */
/*   printf(" I[%3d,%3d,%3d] = %f, weight = %f\n",ix1,iy,iz1, */
/* 	 FourierPt[LookupMap(ix1,iy, iz1)], (x-Xmin) * (Ymax-y) * (z-Zmin) */
/* 	 / ((Xmax-Xmin) * (Ymax-Ymin) * (Zmax-Zmin))); */
/*   printf(" I[%3d,%3d,%3d] = %f, weight = %f\n",ix,iy1,iz, */
/* 	 FourierPt[LookupMap(ix, iy1,iz)  ], (Xmax-x) * (y-Ymin) * (Zmax-z) */
/* 	 / ((Xmax-Xmin) * (Ymax-Ymin) * (Zmax-Zmin))); */
/*   printf(" I[%3d,%3d,%3d] = %f, weight = %f\n",ix,iy1,iz1, */
/* 	 FourierPt[LookupMap(ix, iy1,iz1)], (Xmax-x) * (y-Ymin) * (z-Zmin) */
/* 	 / ((Xmax-Xmin) * (Ymax-Ymin) * (Zmax-Zmin))); */
/*   printf(" I[%3d,%3d,%3d] = %f, weight = %f\n",ix,iy,iz1, */
/* 	 FourierPt[LookupMap(ix, iy, iz1)], (Xmax-x) * (Ymax-y) * (z-Zmin) */
/* 	 / ((Xmax-Xmin) * (Ymax-Ymin) * (Zmax-Zmin))); */
/*   printf(" I[%3d,%3d,%3d] = %f, weight = %f\n",ix1,iy1,iz1, */
/* 	 FourierPt[LookupMap(ix1,iy1,iz1)], (x-Xmin) * (y-Ymin) * (z-Zmin) */
/* 	 / ((Xmax-Xmin) * (Ymax-Ymin) * (Zmax-Zmin))); */
/*   printf(" I[avg] = %f\n", AvgRho);  */

  return AvgRho;
}

/* lookup a point in the Fourier map */
int LookupMap(int ix, int iy, int iz) {
  return ix*mapstep_b*mapstep_c + iy*mapstep_c + ((mapstep_c+ iz % mapstep_c)% mapstep_c);
}
