#define OPENGL 1                     /* define to include openGL code */
#define VERSION "4.2"
#define MAX_ATOM 1000				 /* Maximum number of atoms in asymmetric unit */
#define MAX_EL 200                   /* Maximum number of anisotropic atoms */
#define MAX_POLY 30                  /* Maximum number of polyhedral types */
#define MAX_PLANE 30				 /* Maximum number of plane types */
#define MAX_BOND 50                  /* Maximum number of bond types */
#define MAX_CONE 10                  /* Maximum number of lone pair cone types */
#define MAX_SPHERE 30				 /* Maximum number of sphere types */
#define RAD 57.2957795				 /* Conversion from radians to degrees */
#define MAX_VERTS 20000				 /* Maximum number of generated vertices */
#define MAX_LABELS 1000				 /* Maximum number of textual labels */
#define MAX_ARROW 30                 /* maximum number of arrows */
#define MAX_FRAME 5                  /* Maximum number of frames */
#define MAX_MAP_CONTOUR 10           /* Maximum number of map contours */
#ifndef PI
#define PI 3.1415926
#endif
#define O_OPT 1
#define B_OPT 2
#define P_OPT 4
#define V_OPT 8
#define M_OPT 16
#define L_OPT 32
#define BUFSIZE 512
#ifndef TRUE
#define TRUE 1
#endif
#ifndef FALSE
#define FALSE 0
#endif
#define EPS 1e-7
#define SPIN 1
#define ZOOM 2
#define PAN 3
#define DRAG 4

typedef struct {
	float s;
	float v[3];
} QUAT;

typedef float XY[2];

typedef float XYZ[3];

#ifdef OPENGL
#if defined(__APPLE__) && defined(__MACH__) 
#  include <GLUT/glut.h>
#else
#  include <GL/glut.h>
#  include <GL/gl.h>
#endif
# if defined(FREEGLUT)
#  include <GL/freeglut_ext.h>
# endif 
#endif

#include <stdio.h>
#include <string.h>
#include <math.h>
#include <stdlib.h>
#include <time.h>
#ifdef WIN32
  #include <direct.h>
#endif


#ifndef max
#define max(a,b) ((a) >= (b) ? (a) : (b))
#define min(a,b) ((a) <= (b) ? (a) : (b))
#endif

/* prototype definitions */

void add_to_list (float *, int);
void add_vert (float *, int, int);
void add_vert_nc (float *);
void analyze_bonds (void);
void axeqb (double a[3][3], double b[3], double c[3]);
void axis_to_quaternion(float *axis, float theta, QUAT *quat);
void Blank_Strip (char *);
void build_box_contents (void);
void build_poly_list (int, int);
void Calc_Rot (float *, float *);
int check_atom_name(char *, char *);
void check_color (char *);
void Conv_Sym_Mat (void);
void convert_ellipsoid (void);
float convert_pos(char *);
void crystal_compute (int);
void delay (int);
float determinant (double x[3][3]);
float dist (int, int);
void do_drawing (void);
float dot0_3d(float,float,float,float,float,float,float,float,float);
void draw_cell (int);
void draw_GL_triple(void);
void draw_cursor (void);            /* display openGL cursor */
void Dump_View(void);
float enormsq0_3d(float,float,float,float,float,float);
void expand_atom (int);
int eigen (float *, float *, float *, float a[3][3]);
void Error_Box(char *);
int find_atom(void);
int find_proj_atom(int, int);
void find_all_in_box (int);
void generate_bonds (void);
void generate_cones (void);
void generate_arrows (void);
void generate_ellipsoids (void);
void generate_gl_texts(void);
void generate_planes (void);
void generate_poly (void);
void generate_spheres (void);
void generate_texts (void);
void generate_triple(void);
void get_atom_id (void);
void get_input (int);
void get_label (char *, char *, char *, char *, char *, int);
void import_cif (char *, int, int);
void import_fdat (char *, int, int);
void import_gsas (char *, int, int);
void import_schakal (char *, int, int);
void import_shelx (char *, int, int);
void label_cell(void);
void Locate_Triple (void);
void make_bmat (float *);
int  Maximize_rho(int);
void move_cursor(int, float);
int not_in_slab(float,float,float);
float P_to_C (float);
void Output_Spheres (float *, int);
void plot_vrml_poly (int);
void print_map_info(int);
void print_sym (void);
void Progress_Window(int, char *, float);
void push_cylinder (float df[3], float at[3], float radius, char * color);
void quaternion_to_rotmatrix(QUAT *q, float *m);
void qmult(QUAT *q1, QUAT *q2, QUAT *dest);
void qnormalize(QUAT *q);
void read_fcf(char * infile, int);
void read_flp(char * infile, int);
void read_grd(char * infile, int);
void read_inp (int);
void read_stf(char * infile, int);
void read_vasp(char * infile, int);
void read_w2k(char * infile, int);
void symop (char *);
void show_slab(void);
void tb_init(void);
float tb_project_to_sphere(float, float, float);
void Token_Strip (char *, int);
int toupper (int);
void trackball(void);
void trim_string(char string[], int len);
int Unique_Atom (void);
void Update_Cursor_List(float *, int);
void usage (void);
void vadd(float *src1, float *src2, float *dst);
float vdot(float *v1, float *v2);
void vcross(float *v1, float *v2, float *cross);
void vcopy(float *vsrc, float *vdst);
float vlength(float *v);
void vmult(float *src1, float *src2, float *dst);
float *vnew(float x, float y, float z);
void vnormalize(float *v);
void vprint(float *v);
void vscale(float *v, float f);
void vset(float *v, float x, float y, float z);
void vsub(float *src1, float *src2, float *dst);
void vzero(float *v);
void window_to_tb(float mx, float my, float *x, float *y);
QUAT XYZ_Rot_to_Q(float xrot, float yrot, float zrot);

typedef struct
{
    int info_valid;
    int map_type;
    char title[82];
    float lat_con[6];
    float rhomx;
    float rhomn;
    int map_int[3];
    float xlim[2];
    float ylim[2];
    float zlim[2];
}  MAP_INFO;

typedef struct
{
/* global variables for lattice constants and symmetry operators */

    char spg[3][4];	        /* space group code */
    int ss[24][3][3];           /* rotation operators for symmetry */
    float rss[24][3][3];        /* unitary rotation operators for symmetry */
    float lat_pos[4][3];        /* coordinates of lattice points */
    float ts[24][3];            /* translation operators */
    int sym_op_no[192];         /* saved symmetry operation number */
    float cell_xyz[192][3];     /* storage for generated equiv. positions */
    int sys;                    /* crystal system indicator */
    int nbr;                    /* Bravais lattice indicator */
    int acentric;               /* true in non-centric space group */
    int ng;	                /* number of rotation operations to specify group */
    int nlat;                   /* number of lattice positions */
    int origin1_flag;           /* flag for origin1 space group */
    float origin_offset[3];
    float label_scale;

/* global variables for file names and file variables */

    char savedpath[100];        /* path at start of program */
    char File_Root[100];        /* Input File Name with extension stripped */
    FILE *fpin, *flout;
    FILE *fpoutv;
    FILE *fpoutp;
    char *Str_File_Inp;         /* Input File Name */
    char *Str_File;
    char Pov_File[100];
    char *output;

/* global variables for atom lists */

    int *atom_no;               /* master list atom and symmetry number */
    int *orig_atom_no;          /* saved atom number */
    float atom_xyz[MAX_ATOM][3];/* atomic coordinates */
    char atom_l[MAX_ATOM][4];   /* atom labels */
    int atom_n[MAX_ATOM];       /* input atom number */
    int sv_atom_n[MAX_ATOM];    /* saved input atom number */
    float xyzoff[3];            /* amount to be added to each atom position */

/* cone parameters */

   int ncone;                      /* number of different types of lonepair cones */
   char cone_l1[MAX_CONE][4];      /* label for one end of bond */
   int numlonepairs[MAX_CONE];     /* number of lone pairs to draw at atom */
   float cone_height[MAX_CONE];    /* maximum lone pair distance */
   float cone_min[MAX_CONE];       /* cone tip radius */
   float cone_max[MAX_CONE];       /* cone base radius */
   char col_cone_v[MAX_CONE][40];  /* color and properties of cones for VRML */
   char col_cone_p[MAX_CONE][40];  /* color and properties of cones for POV */
   int cone_fn[MAX_CONE];          /* frame number for cone */

/* magnetic arrow parameters */

  int nmag;                       /* number of magnetic moments */
  int arrow_fn[MAX_ARROW];        /* saved frame number */
  char col_arrow_p[MAX_ARROW][40];/* arrow color */
  char col_arrow_v[MAX_ARROW][40];/* arrow color */
  float arrow_length[MAX_ARROW];  /* length of mag. arrow */
  float arrow_diam[MAX_ARROW];    /* diameter of magnetic arrow */
  float mag_matrix[3][3];         /* transformation between magnetic and nuclear cell */
  float mag_xp[MAX_ARROW][3];     /* position of arrow in real-space cell */
  float mag_xc[MAX_ARROW][3];     /* components of arrow wrt XYZ */

/* polyhedra parameters */

  int npoly;                  /* number of different types of polyhedra */
  int poly_fn[MAX_POLY];      /* frame number for polyhedra commands */
  float polylimit;            /* should be close to zero to allow only regular polygonal faces */
  char poly_l[MAX_POLY][5];   /* polyhedral center label */
  char poly_t[MAX_POLY][5];   /* polyhedral vertex label */
  float poly_size[MAX_POLY];  /* size of polyhedron */
  float poly_min[MAX_POLY];   /* lower limit for shell-like polyhedron */
  char poly_col_v[MAX_POLY][40];/* polyhedron colors for VRML */
  char poly_col_p[MAX_POLY][40];/* polyhedron colors for POV */
  float p_rad_edge[MAX_POLY]; /* individual edge radius for polyhedron */
  char p_col_edge_v[MAX_POLY][40]; /* color of poly edges for VRML */
  char p_col_edge_p[MAX_POLY][40]; /* color of individual poly edges for POV */

/* plane parameters */

  int nplane;                      /* number of different types of planes */
  int plane_fn[MAX_PLANE];         /* frame number for plane commands */
  char plane_col_v[MAX_PLANE][40]; /* plane colors for VRML */
  char plane_col_p[MAX_PLANE][40]; /* plane colors for POV  */
  char plane_l[MAX_PLANE][5];      /* plane center label */
  float plane_size[MAX_PLANE];     /* size of plane coordination */

/* bond parameters */

  int nbond;                       /* number of different types of bonds */
  int bond_fn[MAX_BOND];           /* frame number for bond commands */
  char bond_l1[MAX_BOND][5];       /* label for one end of bond */
  char bond_l2[MAX_BOND][5];       /* label for other end */
  float bond_size[MAX_BOND];       /* bond diameter */
  float bond_min[MAX_BOND];        /* minimum bond distance */
  float bond_max[MAX_BOND];        /* maximum bond distance */
  char col_bond_v[MAX_BOND][40];   /* color and properties of bonds for VRML */
  char col_bond_p[MAX_BOND][40];   /* color and properties of bonds for POV */
  int bond_style[MAX_BOND];	   /* solid bond or dash */
  float Bond_Mult;                 /* Multiplier for Bond Size */
  float BndMult;                   /* Bond multiplier initialization */
  float mol_d;                     /* intramolecular distance for molecular and polyhedral completion */

/* text label parameters */

  int nlabel;                        /* number of textual labels */
  float label_x[MAX_LABELS][3];	     /* position of textual labels */
  char label_label[MAX_LABELS][64];  /* label text */
  int label_fn[MAX_LABELS];          /* frame number */
  int labels_moved;
  int autolabel;                     /* flag for autolabel of SHELX atoms */
  int triple[4];

/* sphere parameters */

  int nsphere;                       /* number of different types of spheres */
  int sphere_fn[MAX_SPHERE];         /* frame number for sphere commands */
  char sphere_l[MAX_SPHERE][5];      /* sphere label */
  int  sphere_n[MAX_SPHERE];         /* atom number - -1 for old behavior */
  float sphere_size[MAX_SPHERE];     /* sphere size */
  char sphere_col_v[MAX_SPHERE][40]; /* sphere colors */
  char sphere_col_p[MAX_SPHERE][40]; /* sphere colors */
  float Sphere_Mult;                 /* Multiplier for Sphere Size */
  float SpMult;                      /* initialization for Sphere_Mult */


/* background parameters */

  char col_bg_p[40];           /* Background color in POV file */
  char col_bg_v[40];           /* Background color in VRML file */
  float glback[3];             /* Background RGB values for openGL */

/* ellipsoid parameters */

  char ellips_col_v[MAX_EL][40]; /* ellipsoid colors for VRML */
  char ellips_col_p[MAX_EL][40]; /* ellipsoid colors for POV */
  float ellips_EV[MAX_EL][3][3]; /* Eigenvectors */
  float ellips_RMS[MAX_EL][3];   /* RMS amplitudes */
  char ellips_l[MAX_EL][5];      /* label for anisotropic atom */
  int ellips_n[MAX_EL];          /* atom number */
  int save_el_number[MAX_EL];    /* save original atom number */
  int ell_type[MAX_EL];          /* form of ellipsoid input */
  float ellips[MAX_EL][6];       /* thermal ellipsoid coefficients */
  float Phong_Value;             /* reflectivity value for Phong highlighting */
  float Phong_Size;              /* Phong highlighting size */
  char Cutout_color[40];         /* color of cutout section */

/* lattice constant variables */
  
  float lat_con[6];          /* lattice constants */
  float rec_lat_con[3];      /* reciprocal lengths */
  double b_mat[3][3];        /* metric tensor */
  float ginv[3][3];          /* direct space metric tensor */
  float b_inv_mat[3][3];      /* inverse metric tensor */

/* variables used in drawing */

    int frame_no;
    int max_frame;              /* maximum number of frames */
    float X_Boxlim;             /* X box limit read from command line */
    float Y_Boxlim;             /* Y box limit read from command line */
    float Z_Boxlim;             /* Z box limit read from command line */
    float X_Origin;             /* X origin read from command line */
    float Y_Origin;             /* Y origin read from command line */
    float Z_Origin;             /* Z origin read from command line */
    float lookat_v1[3], lookat_v2[3];/* "lookat" vectors " */
    int noshadow;               /* True if no shadows for POV objects */
    float ambient;              /* POV lighting options */
    float diffuse;
    float specular;
    float roughness;
    float cryst_lim[MAX_FRAME][6];   /* limits of view box in cell coords */
    float clip_lim[MAX_FRAME][6];    /* limits of clip box in cell coords */
} CRYSTAL;

